# coding: utf-8
import pprint
import six
from enum import Enum



class PaymentAppRefundConfiguration:

    swagger_types = {
    
        'multiple_refunds_supported': 'bool',
        'refund_endpoint': 'str',
        'refund_timeout_in_minutes': 'int',
    }

    attribute_map = {
        'multiple_refunds_supported': 'multipleRefundsSupported','refund_endpoint': 'refundEndpoint','refund_timeout_in_minutes': 'refundTimeoutInMinutes',
    }

    
    _multiple_refunds_supported = None
    _refund_endpoint = None
    _refund_timeout_in_minutes = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.multiple_refunds_supported = kwargs.get('multiple_refunds_supported', None)
        self.refund_endpoint = kwargs.get('refund_endpoint', None)
        self.refund_timeout_in_minutes = kwargs.get('refund_timeout_in_minutes', None)
        

    
    @property
    def multiple_refunds_supported(self):
        """Gets the multiple_refunds_supported of this PaymentAppRefundConfiguration.

            This flag indicates whether the connector supports multiple refunds for a single transaction or not.

        :return: The multiple_refunds_supported of this PaymentAppRefundConfiguration.
        :rtype: bool
        """
        return self._multiple_refunds_supported

    @multiple_refunds_supported.setter
    def multiple_refunds_supported(self, multiple_refunds_supported):
        """Sets the multiple_refunds_supported of this PaymentAppRefundConfiguration.

            This flag indicates whether the connector supports multiple refunds for a single transaction or not.

        :param multiple_refunds_supported: The multiple_refunds_supported of this PaymentAppRefundConfiguration.
        :type: bool
        """

        self._multiple_refunds_supported = multiple_refunds_supported
    
    @property
    def refund_endpoint(self):
        """Gets the refund_endpoint of this PaymentAppRefundConfiguration.

            The refund endpoint is invoked to request the payment service provider to execute a refund.

        :return: The refund_endpoint of this PaymentAppRefundConfiguration.
        :rtype: str
        """
        return self._refund_endpoint

    @refund_endpoint.setter
    def refund_endpoint(self, refund_endpoint):
        """Sets the refund_endpoint of this PaymentAppRefundConfiguration.

            The refund endpoint is invoked to request the payment service provider to execute a refund.

        :param refund_endpoint: The refund_endpoint of this PaymentAppRefundConfiguration.
        :type: str
        """

        self._refund_endpoint = refund_endpoint
    
    @property
    def refund_timeout_in_minutes(self):
        """Gets the refund_timeout_in_minutes of this PaymentAppRefundConfiguration.

            When the refund is triggered we expect a feedback about the state of it. This timeout defines after how long we consider the refund as failed without receiving a final state update.

        :return: The refund_timeout_in_minutes of this PaymentAppRefundConfiguration.
        :rtype: int
        """
        return self._refund_timeout_in_minutes

    @refund_timeout_in_minutes.setter
    def refund_timeout_in_minutes(self, refund_timeout_in_minutes):
        """Sets the refund_timeout_in_minutes of this PaymentAppRefundConfiguration.

            When the refund is triggered we expect a feedback about the state of it. This timeout defines after how long we consider the refund as failed without receiving a final state update.

        :param refund_timeout_in_minutes: The refund_timeout_in_minutes of this PaymentAppRefundConfiguration.
        :type: int
        """

        self._refund_timeout_in_minutes = refund_timeout_in_minutes
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(PaymentAppRefundConfiguration, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, PaymentAppRefundConfiguration):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class CurrencyBankAccount:

    swagger_types = {
    
        'bank_account': 'BankAccount',
        'currency': 'str',
        'environment': 'BankAccountEnvironment',
        'id': 'int',
        'linked_space_id': 'int',
        'version': 'int',
    }

    attribute_map = {
        'bank_account': 'bankAccount','currency': 'currency','environment': 'environment','id': 'id','linked_space_id': 'linkedSpaceId','version': 'version',
    }

    
    _bank_account = None
    _currency = None
    _environment = None
    _id = None
    _linked_space_id = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.bank_account = kwargs.get('bank_account', None)
        self.currency = kwargs.get('currency', None)
        self.environment = kwargs.get('environment', None)
        self.id = kwargs.get('id', None)
        self.linked_space_id = kwargs.get('linked_space_id', None)
        self.version = kwargs.get('version', None)
        

    
    @property
    def bank_account(self):
        """Gets the bank_account of this CurrencyBankAccount.

            

        :return: The bank_account of this CurrencyBankAccount.
        :rtype: BankAccount
        """
        return self._bank_account

    @bank_account.setter
    def bank_account(self, bank_account):
        """Sets the bank_account of this CurrencyBankAccount.

            

        :param bank_account: The bank_account of this CurrencyBankAccount.
        :type: BankAccount
        """

        self._bank_account = bank_account
    
    @property
    def currency(self):
        """Gets the currency of this CurrencyBankAccount.

            

        :return: The currency of this CurrencyBankAccount.
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this CurrencyBankAccount.

            

        :param currency: The currency of this CurrencyBankAccount.
        :type: str
        """

        self._currency = currency
    
    @property
    def environment(self):
        """Gets the environment of this CurrencyBankAccount.

            

        :return: The environment of this CurrencyBankAccount.
        :rtype: BankAccountEnvironment
        """
        return self._environment

    @environment.setter
    def environment(self, environment):
        """Sets the environment of this CurrencyBankAccount.

            

        :param environment: The environment of this CurrencyBankAccount.
        :type: BankAccountEnvironment
        """

        self._environment = environment
    
    @property
    def id(self):
        """Gets the id of this CurrencyBankAccount.

            A unique identifier for the object.

        :return: The id of this CurrencyBankAccount.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this CurrencyBankAccount.

            A unique identifier for the object.

        :param id: The id of this CurrencyBankAccount.
        :type: int
        """

        self._id = id
    
    @property
    def linked_space_id(self):
        """Gets the linked_space_id of this CurrencyBankAccount.

            The ID of the space this object belongs to.

        :return: The linked_space_id of this CurrencyBankAccount.
        :rtype: int
        """
        return self._linked_space_id

    @linked_space_id.setter
    def linked_space_id(self, linked_space_id):
        """Sets the linked_space_id of this CurrencyBankAccount.

            The ID of the space this object belongs to.

        :param linked_space_id: The linked_space_id of this CurrencyBankAccount.
        :type: int
        """

        self._linked_space_id = linked_space_id
    
    @property
    def version(self):
        """Gets the version of this CurrencyBankAccount.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this CurrencyBankAccount.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this CurrencyBankAccount.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this CurrencyBankAccount.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(CurrencyBankAccount, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, CurrencyBankAccount):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class SubscriptionLedgerEntryCreate:

    swagger_types = {
    
        'amount_including_tax': 'float',
        'external_id': 'str',
        'quantity': 'float',
        'subscription_version': 'int',
        'taxes': 'list[TaxCreate]',
        'title': 'str',
    }

    attribute_map = {
        'amount_including_tax': 'amountIncludingTax','external_id': 'externalId','quantity': 'quantity','subscription_version': 'subscriptionVersion','taxes': 'taxes','title': 'title',
    }

    
    _amount_including_tax = None
    _external_id = None
    _quantity = None
    _subscription_version = None
    _taxes = None
    _title = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.amount_including_tax = kwargs.get('amount_including_tax')

        self.external_id = kwargs.get('external_id')

        self.quantity = kwargs.get('quantity')

        self.subscription_version = kwargs.get('subscription_version')

        self.taxes = kwargs.get('taxes', None)
        self.title = kwargs.get('title')

        

    
    @property
    def amount_including_tax(self):
        """Gets the amount_including_tax of this SubscriptionLedgerEntryCreate.

            

        :return: The amount_including_tax of this SubscriptionLedgerEntryCreate.
        :rtype: float
        """
        return self._amount_including_tax

    @amount_including_tax.setter
    def amount_including_tax(self, amount_including_tax):
        """Sets the amount_including_tax of this SubscriptionLedgerEntryCreate.

            

        :param amount_including_tax: The amount_including_tax of this SubscriptionLedgerEntryCreate.
        :type: float
        """
        if amount_including_tax is None:
            raise ValueError("Invalid value for `amount_including_tax`, must not be `None`")

        self._amount_including_tax = amount_including_tax
    
    @property
    def external_id(self):
        """Gets the external_id of this SubscriptionLedgerEntryCreate.

            A client generated nonce which identifies the entity to be created. Subsequent creation requests with the same external ID will not create new entities but return the initially created entity instead.

        :return: The external_id of this SubscriptionLedgerEntryCreate.
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this SubscriptionLedgerEntryCreate.

            A client generated nonce which identifies the entity to be created. Subsequent creation requests with the same external ID will not create new entities but return the initially created entity instead.

        :param external_id: The external_id of this SubscriptionLedgerEntryCreate.
        :type: str
        """
        if external_id is None:
            raise ValueError("Invalid value for `external_id`, must not be `None`")

        self._external_id = external_id
    
    @property
    def quantity(self):
        """Gets the quantity of this SubscriptionLedgerEntryCreate.

            

        :return: The quantity of this SubscriptionLedgerEntryCreate.
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this SubscriptionLedgerEntryCreate.

            

        :param quantity: The quantity of this SubscriptionLedgerEntryCreate.
        :type: float
        """
        if quantity is None:
            raise ValueError("Invalid value for `quantity`, must not be `None`")

        self._quantity = quantity
    
    @property
    def subscription_version(self):
        """Gets the subscription_version of this SubscriptionLedgerEntryCreate.

            

        :return: The subscription_version of this SubscriptionLedgerEntryCreate.
        :rtype: int
        """
        return self._subscription_version

    @subscription_version.setter
    def subscription_version(self, subscription_version):
        """Sets the subscription_version of this SubscriptionLedgerEntryCreate.

            

        :param subscription_version: The subscription_version of this SubscriptionLedgerEntryCreate.
        :type: int
        """
        if subscription_version is None:
            raise ValueError("Invalid value for `subscription_version`, must not be `None`")

        self._subscription_version = subscription_version
    
    @property
    def taxes(self):
        """Gets the taxes of this SubscriptionLedgerEntryCreate.

            

        :return: The taxes of this SubscriptionLedgerEntryCreate.
        :rtype: list[TaxCreate]
        """
        return self._taxes

    @taxes.setter
    def taxes(self, taxes):
        """Sets the taxes of this SubscriptionLedgerEntryCreate.

            

        :param taxes: The taxes of this SubscriptionLedgerEntryCreate.
        :type: list[TaxCreate]
        """

        self._taxes = taxes
    
    @property
    def title(self):
        """Gets the title of this SubscriptionLedgerEntryCreate.

            

        :return: The title of this SubscriptionLedgerEntryCreate.
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this SubscriptionLedgerEntryCreate.

            

        :param title: The title of this SubscriptionLedgerEntryCreate.
        :type: str
        """
        if title is None:
            raise ValueError("Invalid value for `title`, must not be `None`")
        if title is not None and len(title) > 150:
            raise ValueError("Invalid value for `title`, length must be less than or equal to `150`")
        if title is not None and len(title) < 1:
            raise ValueError("Invalid value for `title`, length must be greater than or equal to `1`")

        self._title = title
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(SubscriptionLedgerEntryCreate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, SubscriptionLedgerEntryCreate):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class PaymentInformationHash:

    swagger_types = {
    
        'id': 'int',
        'type': 'PaymentInformationHashType',
        'value': 'str',
    }

    attribute_map = {
        'id': 'id','type': 'type','value': 'value',
    }

    
    _id = None
    _type = None
    _value = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.id = kwargs.get('id', None)
        self.type = kwargs.get('type', None)
        self.value = kwargs.get('value', None)
        

    
    @property
    def id(self):
        """Gets the id of this PaymentInformationHash.

            A unique identifier for the object.

        :return: The id of this PaymentInformationHash.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this PaymentInformationHash.

            A unique identifier for the object.

        :param id: The id of this PaymentInformationHash.
        :type: int
        """

        self._id = id
    
    @property
    def type(self):
        """Gets the type of this PaymentInformationHash.

            

        :return: The type of this PaymentInformationHash.
        :rtype: PaymentInformationHashType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this PaymentInformationHash.

            

        :param type: The type of this PaymentInformationHash.
        :type: PaymentInformationHashType
        """

        self._type = type
    
    @property
    def value(self):
        """Gets the value of this PaymentInformationHash.

            

        :return: The value of this PaymentInformationHash.
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this PaymentInformationHash.

            

        :param value: The value of this PaymentInformationHash.
        :type: str
        """

        self._value = value
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(PaymentInformationHash, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, PaymentInformationHash):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
from enum import Enum, unique


@unique
class PaymentContractState(Enum):
    
    PENDING = "PENDING"
    ACTIVE = "ACTIVE"
    TERMINATING = "TERMINATING"
    TERMINATED = "TERMINATED"
    REJECTED = "REJECTED"




# coding: utf-8
import pprint
import six
from enum import Enum



class ShopifySubscriptionModelItem:

    swagger_types = {
    
        'price_including_tax': 'float',
        'product_id': 'int',
        'quantity': 'float',
        'recalculate_price': 'bool',
        'tax_lines': 'list[ShopifySubscriptionModelTaxLine]',
    }

    attribute_map = {
        'price_including_tax': 'priceIncludingTax','product_id': 'productId','quantity': 'quantity','recalculate_price': 'recalculatePrice','tax_lines': 'taxLines',
    }

    
    _price_including_tax = None
    _product_id = None
    _quantity = None
    _recalculate_price = None
    _tax_lines = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.price_including_tax = kwargs.get('price_including_tax', None)
        self.product_id = kwargs.get('product_id', None)
        self.quantity = kwargs.get('quantity', None)
        self.recalculate_price = kwargs.get('recalculate_price', None)
        self.tax_lines = kwargs.get('tax_lines', None)
        

    
    @property
    def price_including_tax(self):
        """Gets the price_including_tax of this ShopifySubscriptionModelItem.

            

        :return: The price_including_tax of this ShopifySubscriptionModelItem.
        :rtype: float
        """
        return self._price_including_tax

    @price_including_tax.setter
    def price_including_tax(self, price_including_tax):
        """Sets the price_including_tax of this ShopifySubscriptionModelItem.

            

        :param price_including_tax: The price_including_tax of this ShopifySubscriptionModelItem.
        :type: float
        """

        self._price_including_tax = price_including_tax
    
    @property
    def product_id(self):
        """Gets the product_id of this ShopifySubscriptionModelItem.

            

        :return: The product_id of this ShopifySubscriptionModelItem.
        :rtype: int
        """
        return self._product_id

    @product_id.setter
    def product_id(self, product_id):
        """Sets the product_id of this ShopifySubscriptionModelItem.

            

        :param product_id: The product_id of this ShopifySubscriptionModelItem.
        :type: int
        """

        self._product_id = product_id
    
    @property
    def quantity(self):
        """Gets the quantity of this ShopifySubscriptionModelItem.

            

        :return: The quantity of this ShopifySubscriptionModelItem.
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this ShopifySubscriptionModelItem.

            

        :param quantity: The quantity of this ShopifySubscriptionModelItem.
        :type: float
        """

        self._quantity = quantity
    
    @property
    def recalculate_price(self):
        """Gets the recalculate_price of this ShopifySubscriptionModelItem.

            

        :return: The recalculate_price of this ShopifySubscriptionModelItem.
        :rtype: bool
        """
        return self._recalculate_price

    @recalculate_price.setter
    def recalculate_price(self, recalculate_price):
        """Sets the recalculate_price of this ShopifySubscriptionModelItem.

            

        :param recalculate_price: The recalculate_price of this ShopifySubscriptionModelItem.
        :type: bool
        """

        self._recalculate_price = recalculate_price
    
    @property
    def tax_lines(self):
        """Gets the tax_lines of this ShopifySubscriptionModelItem.

            

        :return: The tax_lines of this ShopifySubscriptionModelItem.
        :rtype: list[ShopifySubscriptionModelTaxLine]
        """
        return self._tax_lines

    @tax_lines.setter
    def tax_lines(self, tax_lines):
        """Sets the tax_lines of this ShopifySubscriptionModelItem.

            

        :param tax_lines: The tax_lines of this ShopifySubscriptionModelItem.
        :type: list[ShopifySubscriptionModelTaxLine]
        """

        self._tax_lines = tax_lines
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(ShopifySubscriptionModelItem, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, ShopifySubscriptionModelItem):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class AnalyticsQuery:

    swagger_types = {
    
        'account_id': 'int',
        'external_id': 'str',
        'max_cache_age': 'int',
        'query_string': 'str',
        'scanned_data_limit': 'float',
        'space_ids': 'list[int]',
    }

    attribute_map = {
        'account_id': 'accountId','external_id': 'externalId','max_cache_age': 'maxCacheAge','query_string': 'queryString','scanned_data_limit': 'scannedDataLimit','space_ids': 'spaceIds',
    }

    
    _account_id = None
    _external_id = None
    _max_cache_age = None
    _query_string = None
    _scanned_data_limit = None
    _space_ids = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.account_id = kwargs.get('account_id')

        self.external_id = kwargs.get('external_id', None)
        self.max_cache_age = kwargs.get('max_cache_age', None)
        self.query_string = kwargs.get('query_string', None)
        self.scanned_data_limit = kwargs.get('scanned_data_limit', None)
        self.space_ids = kwargs.get('space_ids', None)
        

    
    @property
    def account_id(self):
        """Gets the account_id of this AnalyticsQuery.

            The mandatory ID of an account in which the query shall be executed. Must be a valid account ID greater than 0.

        :return: The account_id of this AnalyticsQuery.
        :rtype: int
        """
        return self._account_id

    @account_id.setter
    def account_id(self, account_id):
        """Sets the account_id of this AnalyticsQuery.

            The mandatory ID of an account in which the query shall be executed. Must be a valid account ID greater than 0.

        :param account_id: The account_id of this AnalyticsQuery.
        :type: int
        """
        if account_id is None:
            raise ValueError("Invalid value for `account_id`, must not be `None`")

        self._account_id = account_id
    
    @property
    def external_id(self):
        """Gets the external_id of this AnalyticsQuery.

            A client generated nonce which uniquely identifies the query to be executed. Subsequent submissions with the same external ID will not re-execute the query but instead return the existing execution with that ID. Either the External ID or a Maximal Cache Age greater than 0 must be specified. If both are specified the External ID will have precedence and the Maximal Cache Age will be ignored.

        :return: The external_id of this AnalyticsQuery.
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this AnalyticsQuery.

            A client generated nonce which uniquely identifies the query to be executed. Subsequent submissions with the same external ID will not re-execute the query but instead return the existing execution with that ID. Either the External ID or a Maximal Cache Age greater than 0 must be specified. If both are specified the External ID will have precedence and the Maximal Cache Age will be ignored.

        :param external_id: The external_id of this AnalyticsQuery.
        :type: str
        """

        self._external_id = external_id
    
    @property
    def max_cache_age(self):
        """Gets the max_cache_age of this AnalyticsQuery.

            The maximal age in minutes of cached query executions to return. If an equivalent query execution with the same Query String, Account ID and Spaces parameters not older than the specified age is already available that execution will be returned instead of a newly started execution. Set to 0 or null (and set a unique, previously unused External ID) to force a new query execution irrespective of previous executions. Either the External ID or a Cache Duration greater than 0 must be specified. If both are specified, the External ID will be preferred (and the Maximal Cache Age ignored).

        :return: The max_cache_age of this AnalyticsQuery.
        :rtype: int
        """
        return self._max_cache_age

    @max_cache_age.setter
    def max_cache_age(self, max_cache_age):
        """Sets the max_cache_age of this AnalyticsQuery.

            The maximal age in minutes of cached query executions to return. If an equivalent query execution with the same Query String, Account ID and Spaces parameters not older than the specified age is already available that execution will be returned instead of a newly started execution. Set to 0 or null (and set a unique, previously unused External ID) to force a new query execution irrespective of previous executions. Either the External ID or a Cache Duration greater than 0 must be specified. If both are specified, the External ID will be preferred (and the Maximal Cache Age ignored).

        :param max_cache_age: The max_cache_age of this AnalyticsQuery.
        :type: int
        """

        self._max_cache_age = max_cache_age
    
    @property
    def query_string(self):
        """Gets the query_string of this AnalyticsQuery.

            The SQL statement which is being submitted for execution. Must be a valid PrestoDB/Athena SQL statement.

        :return: The query_string of this AnalyticsQuery.
        :rtype: str
        """
        return self._query_string

    @query_string.setter
    def query_string(self, query_string):
        """Sets the query_string of this AnalyticsQuery.

            The SQL statement which is being submitted for execution. Must be a valid PrestoDB/Athena SQL statement.

        :param query_string: The query_string of this AnalyticsQuery.
        :type: str
        """
        if query_string is not None and len(query_string) > 4096:
            raise ValueError("Invalid value for `query_string`, length must be less than or equal to `4096`")
        if query_string is not None and len(query_string) < 1:
            raise ValueError("Invalid value for `query_string`, length must be greater than or equal to `1`")

        self._query_string = query_string
    
    @property
    def scanned_data_limit(self):
        """Gets the scanned_data_limit of this AnalyticsQuery.

            The maximal amount of scanned data that this query is allowed to scan. After this limit is reached query will be canceled by the system. 

        :return: The scanned_data_limit of this AnalyticsQuery.
        :rtype: float
        """
        return self._scanned_data_limit

    @scanned_data_limit.setter
    def scanned_data_limit(self, scanned_data_limit):
        """Sets the scanned_data_limit of this AnalyticsQuery.

            The maximal amount of scanned data that this query is allowed to scan. After this limit is reached query will be canceled by the system. 

        :param scanned_data_limit: The scanned_data_limit of this AnalyticsQuery.
        :type: float
        """

        self._scanned_data_limit = scanned_data_limit
    
    @property
    def space_ids(self):
        """Gets the space_ids of this AnalyticsQuery.

            The IDs of the spaces in which the query shall be executed. At most 5 space IDs may be specified. All specified spaces must be owned by the account specified by the accountId property. The spaces property may be missing or empty to query all spaces of the specified account.

        :return: The space_ids of this AnalyticsQuery.
        :rtype: list[int]
        """
        return self._space_ids

    @space_ids.setter
    def space_ids(self, space_ids):
        """Sets the space_ids of this AnalyticsQuery.

            The IDs of the spaces in which the query shall be executed. At most 5 space IDs may be specified. All specified spaces must be owned by the account specified by the accountId property. The spaces property may be missing or empty to query all spaces of the specified account.

        :param space_ids: The space_ids of this AnalyticsQuery.
        :type: list[int]
        """

        self._space_ids = space_ids
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(AnalyticsQuery, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, AnalyticsQuery):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class LineItem:

    swagger_types = {
    
        'aggregated_tax_rate': 'float',
        'amount_excluding_tax': 'float',
        'amount_including_tax': 'float',
        'attributes': 'dict(str, LineItemAttribute)',
        'discount_excluding_tax': 'float',
        'discount_including_tax': 'float',
        'name': 'str',
        'quantity': 'float',
        'shipping_required': 'bool',
        'sku': 'str',
        'tax_amount': 'float',
        'tax_amount_per_unit': 'float',
        'taxes': 'list[Tax]',
        'type': 'LineItemType',
        'undiscounted_amount_excluding_tax': 'float',
        'undiscounted_amount_including_tax': 'float',
        'undiscounted_unit_price_excluding_tax': 'float',
        'undiscounted_unit_price_including_tax': 'float',
        'unique_id': 'str',
        'unit_price_excluding_tax': 'float',
        'unit_price_including_tax': 'float',
    }

    attribute_map = {
        'aggregated_tax_rate': 'aggregatedTaxRate','amount_excluding_tax': 'amountExcludingTax','amount_including_tax': 'amountIncludingTax','attributes': 'attributes','discount_excluding_tax': 'discountExcludingTax','discount_including_tax': 'discountIncludingTax','name': 'name','quantity': 'quantity','shipping_required': 'shippingRequired','sku': 'sku','tax_amount': 'taxAmount','tax_amount_per_unit': 'taxAmountPerUnit','taxes': 'taxes','type': 'type','undiscounted_amount_excluding_tax': 'undiscountedAmountExcludingTax','undiscounted_amount_including_tax': 'undiscountedAmountIncludingTax','undiscounted_unit_price_excluding_tax': 'undiscountedUnitPriceExcludingTax','undiscounted_unit_price_including_tax': 'undiscountedUnitPriceIncludingTax','unique_id': 'uniqueId','unit_price_excluding_tax': 'unitPriceExcludingTax','unit_price_including_tax': 'unitPriceIncludingTax',
    }

    
    _aggregated_tax_rate = None
    _amount_excluding_tax = None
    _amount_including_tax = None
    _attributes = None
    _discount_excluding_tax = None
    _discount_including_tax = None
    _name = None
    _quantity = None
    _shipping_required = None
    _sku = None
    _tax_amount = None
    _tax_amount_per_unit = None
    _taxes = None
    _type = None
    _undiscounted_amount_excluding_tax = None
    _undiscounted_amount_including_tax = None
    _undiscounted_unit_price_excluding_tax = None
    _undiscounted_unit_price_including_tax = None
    _unique_id = None
    _unit_price_excluding_tax = None
    _unit_price_including_tax = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.aggregated_tax_rate = kwargs.get('aggregated_tax_rate', None)
        self.amount_excluding_tax = kwargs.get('amount_excluding_tax', None)
        self.amount_including_tax = kwargs.get('amount_including_tax', None)
        self.attributes = kwargs.get('attributes', None)
        self.discount_excluding_tax = kwargs.get('discount_excluding_tax', None)
        self.discount_including_tax = kwargs.get('discount_including_tax', None)
        self.name = kwargs.get('name', None)
        self.quantity = kwargs.get('quantity', None)
        self.shipping_required = kwargs.get('shipping_required', None)
        self.sku = kwargs.get('sku', None)
        self.tax_amount = kwargs.get('tax_amount', None)
        self.tax_amount_per_unit = kwargs.get('tax_amount_per_unit', None)
        self.taxes = kwargs.get('taxes', None)
        self.type = kwargs.get('type', None)
        self.undiscounted_amount_excluding_tax = kwargs.get('undiscounted_amount_excluding_tax', None)
        self.undiscounted_amount_including_tax = kwargs.get('undiscounted_amount_including_tax', None)
        self.undiscounted_unit_price_excluding_tax = kwargs.get('undiscounted_unit_price_excluding_tax', None)
        self.undiscounted_unit_price_including_tax = kwargs.get('undiscounted_unit_price_including_tax', None)
        self.unique_id = kwargs.get('unique_id', None)
        self.unit_price_excluding_tax = kwargs.get('unit_price_excluding_tax', None)
        self.unit_price_including_tax = kwargs.get('unit_price_including_tax', None)
        

    
    @property
    def aggregated_tax_rate(self):
        """Gets the aggregated_tax_rate of this LineItem.

            The aggregated tax rate is the sum of all tax rates of the line item.

        :return: The aggregated_tax_rate of this LineItem.
        :rtype: float
        """
        return self._aggregated_tax_rate

    @aggregated_tax_rate.setter
    def aggregated_tax_rate(self, aggregated_tax_rate):
        """Sets the aggregated_tax_rate of this LineItem.

            The aggregated tax rate is the sum of all tax rates of the line item.

        :param aggregated_tax_rate: The aggregated_tax_rate of this LineItem.
        :type: float
        """

        self._aggregated_tax_rate = aggregated_tax_rate
    
    @property
    def amount_excluding_tax(self):
        """Gets the amount_excluding_tax of this LineItem.

            

        :return: The amount_excluding_tax of this LineItem.
        :rtype: float
        """
        return self._amount_excluding_tax

    @amount_excluding_tax.setter
    def amount_excluding_tax(self, amount_excluding_tax):
        """Sets the amount_excluding_tax of this LineItem.

            

        :param amount_excluding_tax: The amount_excluding_tax of this LineItem.
        :type: float
        """

        self._amount_excluding_tax = amount_excluding_tax
    
    @property
    def amount_including_tax(self):
        """Gets the amount_including_tax of this LineItem.

            

        :return: The amount_including_tax of this LineItem.
        :rtype: float
        """
        return self._amount_including_tax

    @amount_including_tax.setter
    def amount_including_tax(self, amount_including_tax):
        """Sets the amount_including_tax of this LineItem.

            

        :param amount_including_tax: The amount_including_tax of this LineItem.
        :type: float
        """

        self._amount_including_tax = amount_including_tax
    
    @property
    def attributes(self):
        """Gets the attributes of this LineItem.

            

        :return: The attributes of this LineItem.
        :rtype: dict(str, LineItemAttribute)
        """
        return self._attributes

    @attributes.setter
    def attributes(self, attributes):
        """Sets the attributes of this LineItem.

            

        :param attributes: The attributes of this LineItem.
        :type: dict(str, LineItemAttribute)
        """

        self._attributes = attributes
    
    @property
    def discount_excluding_tax(self):
        """Gets the discount_excluding_tax of this LineItem.

            

        :return: The discount_excluding_tax of this LineItem.
        :rtype: float
        """
        return self._discount_excluding_tax

    @discount_excluding_tax.setter
    def discount_excluding_tax(self, discount_excluding_tax):
        """Sets the discount_excluding_tax of this LineItem.

            

        :param discount_excluding_tax: The discount_excluding_tax of this LineItem.
        :type: float
        """

        self._discount_excluding_tax = discount_excluding_tax
    
    @property
    def discount_including_tax(self):
        """Gets the discount_including_tax of this LineItem.

            

        :return: The discount_including_tax of this LineItem.
        :rtype: float
        """
        return self._discount_including_tax

    @discount_including_tax.setter
    def discount_including_tax(self, discount_including_tax):
        """Sets the discount_including_tax of this LineItem.

            

        :param discount_including_tax: The discount_including_tax of this LineItem.
        :type: float
        """

        self._discount_including_tax = discount_including_tax
    
    @property
    def name(self):
        """Gets the name of this LineItem.

            

        :return: The name of this LineItem.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this LineItem.

            

        :param name: The name of this LineItem.
        :type: str
        """
        if name is not None and len(name) > 150:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `150`")
        if name is not None and len(name) < 1:
            raise ValueError("Invalid value for `name`, length must be greater than or equal to `1`")

        self._name = name
    
    @property
    def quantity(self):
        """Gets the quantity of this LineItem.

            

        :return: The quantity of this LineItem.
        :rtype: float
        """
        return self._quantity

    @quantity.setter
    def quantity(self, quantity):
        """Sets the quantity of this LineItem.

            

        :param quantity: The quantity of this LineItem.
        :type: float
        """

        self._quantity = quantity
    
    @property
    def shipping_required(self):
        """Gets the shipping_required of this LineItem.

            

        :return: The shipping_required of this LineItem.
        :rtype: bool
        """
        return self._shipping_required

    @shipping_required.setter
    def shipping_required(self, shipping_required):
        """Sets the shipping_required of this LineItem.

            

        :param shipping_required: The shipping_required of this LineItem.
        :type: bool
        """

        self._shipping_required = shipping_required
    
    @property
    def sku(self):
        """Gets the sku of this LineItem.

            

        :return: The sku of this LineItem.
        :rtype: str
        """
        return self._sku

    @sku.setter
    def sku(self, sku):
        """Sets the sku of this LineItem.

            

        :param sku: The sku of this LineItem.
        :type: str
        """
        if sku is not None and len(sku) > 200:
            raise ValueError("Invalid value for `sku`, length must be less than or equal to `200`")

        self._sku = sku
    
    @property
    def tax_amount(self):
        """Gets the tax_amount of this LineItem.

            

        :return: The tax_amount of this LineItem.
        :rtype: float
        """
        return self._tax_amount

    @tax_amount.setter
    def tax_amount(self, tax_amount):
        """Sets the tax_amount of this LineItem.

            

        :param tax_amount: The tax_amount of this LineItem.
        :type: float
        """

        self._tax_amount = tax_amount
    
    @property
    def tax_amount_per_unit(self):
        """Gets the tax_amount_per_unit of this LineItem.

            

        :return: The tax_amount_per_unit of this LineItem.
        :rtype: float
        """
        return self._tax_amount_per_unit

    @tax_amount_per_unit.setter
    def tax_amount_per_unit(self, tax_amount_per_unit):
        """Sets the tax_amount_per_unit of this LineItem.

            

        :param tax_amount_per_unit: The tax_amount_per_unit of this LineItem.
        :type: float
        """

        self._tax_amount_per_unit = tax_amount_per_unit
    
    @property
    def taxes(self):
        """Gets the taxes of this LineItem.

            

        :return: The taxes of this LineItem.
        :rtype: list[Tax]
        """
        return self._taxes

    @taxes.setter
    def taxes(self, taxes):
        """Sets the taxes of this LineItem.

            

        :param taxes: The taxes of this LineItem.
        :type: list[Tax]
        """

        self._taxes = taxes
    
    @property
    def type(self):
        """Gets the type of this LineItem.

            

        :return: The type of this LineItem.
        :rtype: LineItemType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this LineItem.

            

        :param type: The type of this LineItem.
        :type: LineItemType
        """

        self._type = type
    
    @property
    def undiscounted_amount_excluding_tax(self):
        """Gets the undiscounted_amount_excluding_tax of this LineItem.

            

        :return: The undiscounted_amount_excluding_tax of this LineItem.
        :rtype: float
        """
        return self._undiscounted_amount_excluding_tax

    @undiscounted_amount_excluding_tax.setter
    def undiscounted_amount_excluding_tax(self, undiscounted_amount_excluding_tax):
        """Sets the undiscounted_amount_excluding_tax of this LineItem.

            

        :param undiscounted_amount_excluding_tax: The undiscounted_amount_excluding_tax of this LineItem.
        :type: float
        """

        self._undiscounted_amount_excluding_tax = undiscounted_amount_excluding_tax
    
    @property
    def undiscounted_amount_including_tax(self):
        """Gets the undiscounted_amount_including_tax of this LineItem.

            

        :return: The undiscounted_amount_including_tax of this LineItem.
        :rtype: float
        """
        return self._undiscounted_amount_including_tax

    @undiscounted_amount_including_tax.setter
    def undiscounted_amount_including_tax(self, undiscounted_amount_including_tax):
        """Sets the undiscounted_amount_including_tax of this LineItem.

            

        :param undiscounted_amount_including_tax: The undiscounted_amount_including_tax of this LineItem.
        :type: float
        """

        self._undiscounted_amount_including_tax = undiscounted_amount_including_tax
    
    @property
    def undiscounted_unit_price_excluding_tax(self):
        """Gets the undiscounted_unit_price_excluding_tax of this LineItem.

            

        :return: The undiscounted_unit_price_excluding_tax of this LineItem.
        :rtype: float
        """
        return self._undiscounted_unit_price_excluding_tax

    @undiscounted_unit_price_excluding_tax.setter
    def undiscounted_unit_price_excluding_tax(self, undiscounted_unit_price_excluding_tax):
        """Sets the undiscounted_unit_price_excluding_tax of this LineItem.

            

        :param undiscounted_unit_price_excluding_tax: The undiscounted_unit_price_excluding_tax of this LineItem.
        :type: float
        """

        self._undiscounted_unit_price_excluding_tax = undiscounted_unit_price_excluding_tax
    
    @property
    def undiscounted_unit_price_including_tax(self):
        """Gets the undiscounted_unit_price_including_tax of this LineItem.

            

        :return: The undiscounted_unit_price_including_tax of this LineItem.
        :rtype: float
        """
        return self._undiscounted_unit_price_including_tax

    @undiscounted_unit_price_including_tax.setter
    def undiscounted_unit_price_including_tax(self, undiscounted_unit_price_including_tax):
        """Sets the undiscounted_unit_price_including_tax of this LineItem.

            

        :param undiscounted_unit_price_including_tax: The undiscounted_unit_price_including_tax of this LineItem.
        :type: float
        """

        self._undiscounted_unit_price_including_tax = undiscounted_unit_price_including_tax
    
    @property
    def unique_id(self):
        """Gets the unique_id of this LineItem.

            The unique id identifies the line item within the set of line items associated with the transaction.

        :return: The unique_id of this LineItem.
        :rtype: str
        """
        return self._unique_id

    @unique_id.setter
    def unique_id(self, unique_id):
        """Sets the unique_id of this LineItem.

            The unique id identifies the line item within the set of line items associated with the transaction.

        :param unique_id: The unique_id of this LineItem.
        :type: str
        """
        if unique_id is not None and len(unique_id) > 200:
            raise ValueError("Invalid value for `unique_id`, length must be less than or equal to `200`")

        self._unique_id = unique_id
    
    @property
    def unit_price_excluding_tax(self):
        """Gets the unit_price_excluding_tax of this LineItem.

            

        :return: The unit_price_excluding_tax of this LineItem.
        :rtype: float
        """
        return self._unit_price_excluding_tax

    @unit_price_excluding_tax.setter
    def unit_price_excluding_tax(self, unit_price_excluding_tax):
        """Sets the unit_price_excluding_tax of this LineItem.

            

        :param unit_price_excluding_tax: The unit_price_excluding_tax of this LineItem.
        :type: float
        """

        self._unit_price_excluding_tax = unit_price_excluding_tax
    
    @property
    def unit_price_including_tax(self):
        """Gets the unit_price_including_tax of this LineItem.

            

        :return: The unit_price_including_tax of this LineItem.
        :rtype: float
        """
        return self._unit_price_including_tax

    @unit_price_including_tax.setter
    def unit_price_including_tax(self, unit_price_including_tax):
        """Sets the unit_price_including_tax of this LineItem.

            

        :param unit_price_including_tax: The unit_price_including_tax of this LineItem.
        :type: float
        """

        self._unit_price_including_tax = unit_price_including_tax
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(LineItem, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, LineItem):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum
from . import TransactionAwareEntity


class InvoiceReconciliationRecord(TransactionAwareEntity):

    swagger_types = {
    
        'address': 'str',
        'amount': 'float',
        'city': 'str',
        'country': 'str',
        'created_on': 'datetime',
        'currency': 'str',
        'discarded_by': 'int',
        'discarded_on': 'datetime',
        'environment': 'ChargeAttemptEnvironment',
        'family_name': 'str',
        'given_name': 'str',
        'iban': 'str',
        'last_resolution_failure': 'FailureReason',
        'participant_number': 'str',
        'payment_fee_amount': 'float',
        'payment_fee_currency': 'str',
        'payment_reason': 'str',
        'planned_purge_date': 'datetime',
        'post_code': 'str',
        'reference_number': 'str',
        'rejection_status': 'InvoiceReconciliationRecordRejectionStatus',
        'resolved_by': 'int',
        'resolved_on': 'datetime',
        'sender_bank_account': 'str',
        'state': 'InvoiceReconciliationRecordState',
        'street': 'str',
        'type': 'InvoiceReconciliationRecordType',
        'unique_id': 'str',
        'value_date': 'datetime',
        'version': 'int',
    }

    attribute_map = {
        'address': 'address','amount': 'amount','city': 'city','country': 'country','created_on': 'createdOn','currency': 'currency','discarded_by': 'discardedBy','discarded_on': 'discardedOn','environment': 'environment','family_name': 'familyName','given_name': 'givenName','iban': 'iban','last_resolution_failure': 'lastResolutionFailure','participant_number': 'participantNumber','payment_fee_amount': 'paymentFeeAmount','payment_fee_currency': 'paymentFeeCurrency','payment_reason': 'paymentReason','planned_purge_date': 'plannedPurgeDate','post_code': 'postCode','reference_number': 'referenceNumber','rejection_status': 'rejectionStatus','resolved_by': 'resolvedBy','resolved_on': 'resolvedOn','sender_bank_account': 'senderBankAccount','state': 'state','street': 'street','type': 'type','unique_id': 'uniqueId','value_date': 'valueDate','version': 'version',
    }

    
    _address = None
    _amount = None
    _city = None
    _country = None
    _created_on = None
    _currency = None
    _discarded_by = None
    _discarded_on = None
    _environment = None
    _family_name = None
    _given_name = None
    _iban = None
    _last_resolution_failure = None
    _participant_number = None
    _payment_fee_amount = None
    _payment_fee_currency = None
    _payment_reason = None
    _planned_purge_date = None
    _post_code = None
    _reference_number = None
    _rejection_status = None
    _resolved_by = None
    _resolved_on = None
    _sender_bank_account = None
    _state = None
    _street = None
    _type = None
    _unique_id = None
    _value_date = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.address = kwargs.get('address', None)
        self.amount = kwargs.get('amount', None)
        self.city = kwargs.get('city', None)
        self.country = kwargs.get('country', None)
        self.created_on = kwargs.get('created_on', None)
        self.currency = kwargs.get('currency', None)
        self.discarded_by = kwargs.get('discarded_by', None)
        self.discarded_on = kwargs.get('discarded_on', None)
        self.environment = kwargs.get('environment', None)
        self.family_name = kwargs.get('family_name', None)
        self.given_name = kwargs.get('given_name', None)
        self.iban = kwargs.get('iban', None)
        self.last_resolution_failure = kwargs.get('last_resolution_failure', None)
        self.participant_number = kwargs.get('participant_number', None)
        self.payment_fee_amount = kwargs.get('payment_fee_amount', None)
        self.payment_fee_currency = kwargs.get('payment_fee_currency', None)
        self.payment_reason = kwargs.get('payment_reason', None)
        self.planned_purge_date = kwargs.get('planned_purge_date', None)
        self.post_code = kwargs.get('post_code', None)
        self.reference_number = kwargs.get('reference_number', None)
        self.rejection_status = kwargs.get('rejection_status', None)
        self.resolved_by = kwargs.get('resolved_by', None)
        self.resolved_on = kwargs.get('resolved_on', None)
        self.sender_bank_account = kwargs.get('sender_bank_account', None)
        self.state = kwargs.get('state', None)
        self.street = kwargs.get('street', None)
        self.type = kwargs.get('type', None)
        self.unique_id = kwargs.get('unique_id', None)
        self.value_date = kwargs.get('value_date', None)
        self.version = kwargs.get('version', None)
        super().__init__(**kwargs)
        self.swagger_types.update(super().swagger_types)
        self.attribute_map.update(super().attribute_map)

    
    @property
    def address(self):
        """Gets the address of this InvoiceReconciliationRecord.

            

        :return: The address of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._address

    @address.setter
    def address(self, address):
        """Sets the address of this InvoiceReconciliationRecord.

            

        :param address: The address of this InvoiceReconciliationRecord.
        :type: str
        """

        self._address = address
    
    @property
    def amount(self):
        """Gets the amount of this InvoiceReconciliationRecord.

            

        :return: The amount of this InvoiceReconciliationRecord.
        :rtype: float
        """
        return self._amount

    @amount.setter
    def amount(self, amount):
        """Sets the amount of this InvoiceReconciliationRecord.

            

        :param amount: The amount of this InvoiceReconciliationRecord.
        :type: float
        """

        self._amount = amount
    
    @property
    def city(self):
        """Gets the city of this InvoiceReconciliationRecord.

            

        :return: The city of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._city

    @city.setter
    def city(self, city):
        """Sets the city of this InvoiceReconciliationRecord.

            

        :param city: The city of this InvoiceReconciliationRecord.
        :type: str
        """

        self._city = city
    
    @property
    def country(self):
        """Gets the country of this InvoiceReconciliationRecord.

            

        :return: The country of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._country

    @country.setter
    def country(self, country):
        """Sets the country of this InvoiceReconciliationRecord.

            

        :param country: The country of this InvoiceReconciliationRecord.
        :type: str
        """

        self._country = country
    
    @property
    def created_on(self):
        """Gets the created_on of this InvoiceReconciliationRecord.

            The date and time when the object was created.

        :return: The created_on of this InvoiceReconciliationRecord.
        :rtype: datetime
        """
        return self._created_on

    @created_on.setter
    def created_on(self, created_on):
        """Sets the created_on of this InvoiceReconciliationRecord.

            The date and time when the object was created.

        :param created_on: The created_on of this InvoiceReconciliationRecord.
        :type: datetime
        """

        self._created_on = created_on
    
    @property
    def currency(self):
        """Gets the currency of this InvoiceReconciliationRecord.

            

        :return: The currency of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._currency

    @currency.setter
    def currency(self, currency):
        """Sets the currency of this InvoiceReconciliationRecord.

            

        :param currency: The currency of this InvoiceReconciliationRecord.
        :type: str
        """

        self._currency = currency
    
    @property
    def discarded_by(self):
        """Gets the discarded_by of this InvoiceReconciliationRecord.

            

        :return: The discarded_by of this InvoiceReconciliationRecord.
        :rtype: int
        """
        return self._discarded_by

    @discarded_by.setter
    def discarded_by(self, discarded_by):
        """Sets the discarded_by of this InvoiceReconciliationRecord.

            

        :param discarded_by: The discarded_by of this InvoiceReconciliationRecord.
        :type: int
        """

        self._discarded_by = discarded_by
    
    @property
    def discarded_on(self):
        """Gets the discarded_on of this InvoiceReconciliationRecord.

            The discarded on date indicates when the bank transaction has been discarded.

        :return: The discarded_on of this InvoiceReconciliationRecord.
        :rtype: datetime
        """
        return self._discarded_on

    @discarded_on.setter
    def discarded_on(self, discarded_on):
        """Sets the discarded_on of this InvoiceReconciliationRecord.

            The discarded on date indicates when the bank transaction has been discarded.

        :param discarded_on: The discarded_on of this InvoiceReconciliationRecord.
        :type: datetime
        """

        self._discarded_on = discarded_on
    
    @property
    def environment(self):
        """Gets the environment of this InvoiceReconciliationRecord.

            

        :return: The environment of this InvoiceReconciliationRecord.
        :rtype: ChargeAttemptEnvironment
        """
        return self._environment

    @environment.setter
    def environment(self, environment):
        """Sets the environment of this InvoiceReconciliationRecord.

            

        :param environment: The environment of this InvoiceReconciliationRecord.
        :type: ChargeAttemptEnvironment
        """

        self._environment = environment
    
    @property
    def family_name(self):
        """Gets the family_name of this InvoiceReconciliationRecord.

            

        :return: The family_name of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._family_name

    @family_name.setter
    def family_name(self, family_name):
        """Sets the family_name of this InvoiceReconciliationRecord.

            

        :param family_name: The family_name of this InvoiceReconciliationRecord.
        :type: str
        """

        self._family_name = family_name
    
    @property
    def given_name(self):
        """Gets the given_name of this InvoiceReconciliationRecord.

            

        :return: The given_name of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._given_name

    @given_name.setter
    def given_name(self, given_name):
        """Sets the given_name of this InvoiceReconciliationRecord.

            

        :param given_name: The given_name of this InvoiceReconciliationRecord.
        :type: str
        """

        self._given_name = given_name
    
    @property
    def iban(self):
        """Gets the iban of this InvoiceReconciliationRecord.

            

        :return: The iban of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._iban

    @iban.setter
    def iban(self, iban):
        """Sets the iban of this InvoiceReconciliationRecord.

            

        :param iban: The iban of this InvoiceReconciliationRecord.
        :type: str
        """
        if iban is not None and len(iban) > 100:
            raise ValueError("Invalid value for `iban`, length must be less than or equal to `100`")

        self._iban = iban
    
    @property
    def last_resolution_failure(self):
        """Gets the last_resolution_failure of this InvoiceReconciliationRecord.

            

        :return: The last_resolution_failure of this InvoiceReconciliationRecord.
        :rtype: FailureReason
        """
        return self._last_resolution_failure

    @last_resolution_failure.setter
    def last_resolution_failure(self, last_resolution_failure):
        """Sets the last_resolution_failure of this InvoiceReconciliationRecord.

            

        :param last_resolution_failure: The last_resolution_failure of this InvoiceReconciliationRecord.
        :type: FailureReason
        """

        self._last_resolution_failure = last_resolution_failure
    
    @property
    def participant_number(self):
        """Gets the participant_number of this InvoiceReconciliationRecord.

            

        :return: The participant_number of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._participant_number

    @participant_number.setter
    def participant_number(self, participant_number):
        """Sets the participant_number of this InvoiceReconciliationRecord.

            

        :param participant_number: The participant_number of this InvoiceReconciliationRecord.
        :type: str
        """
        if participant_number is not None and len(participant_number) > 100:
            raise ValueError("Invalid value for `participant_number`, length must be less than or equal to `100`")

        self._participant_number = participant_number
    
    @property
    def payment_fee_amount(self):
        """Gets the payment_fee_amount of this InvoiceReconciliationRecord.

            

        :return: The payment_fee_amount of this InvoiceReconciliationRecord.
        :rtype: float
        """
        return self._payment_fee_amount

    @payment_fee_amount.setter
    def payment_fee_amount(self, payment_fee_amount):
        """Sets the payment_fee_amount of this InvoiceReconciliationRecord.

            

        :param payment_fee_amount: The payment_fee_amount of this InvoiceReconciliationRecord.
        :type: float
        """

        self._payment_fee_amount = payment_fee_amount
    
    @property
    def payment_fee_currency(self):
        """Gets the payment_fee_currency of this InvoiceReconciliationRecord.

            

        :return: The payment_fee_currency of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._payment_fee_currency

    @payment_fee_currency.setter
    def payment_fee_currency(self, payment_fee_currency):
        """Sets the payment_fee_currency of this InvoiceReconciliationRecord.

            

        :param payment_fee_currency: The payment_fee_currency of this InvoiceReconciliationRecord.
        :type: str
        """

        self._payment_fee_currency = payment_fee_currency
    
    @property
    def payment_reason(self):
        """Gets the payment_reason of this InvoiceReconciliationRecord.

            

        :return: The payment_reason of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._payment_reason

    @payment_reason.setter
    def payment_reason(self, payment_reason):
        """Sets the payment_reason of this InvoiceReconciliationRecord.

            

        :param payment_reason: The payment_reason of this InvoiceReconciliationRecord.
        :type: str
        """

        self._payment_reason = payment_reason
    
    @property
    def planned_purge_date(self):
        """Gets the planned_purge_date of this InvoiceReconciliationRecord.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :return: The planned_purge_date of this InvoiceReconciliationRecord.
        :rtype: datetime
        """
        return self._planned_purge_date

    @planned_purge_date.setter
    def planned_purge_date(self, planned_purge_date):
        """Sets the planned_purge_date of this InvoiceReconciliationRecord.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :param planned_purge_date: The planned_purge_date of this InvoiceReconciliationRecord.
        :type: datetime
        """

        self._planned_purge_date = planned_purge_date
    
    @property
    def post_code(self):
        """Gets the post_code of this InvoiceReconciliationRecord.

            

        :return: The post_code of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._post_code

    @post_code.setter
    def post_code(self, post_code):
        """Sets the post_code of this InvoiceReconciliationRecord.

            

        :param post_code: The post_code of this InvoiceReconciliationRecord.
        :type: str
        """

        self._post_code = post_code
    
    @property
    def reference_number(self):
        """Gets the reference_number of this InvoiceReconciliationRecord.

            

        :return: The reference_number of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._reference_number

    @reference_number.setter
    def reference_number(self, reference_number):
        """Sets the reference_number of this InvoiceReconciliationRecord.

            

        :param reference_number: The reference_number of this InvoiceReconciliationRecord.
        :type: str
        """
        if reference_number is not None and len(reference_number) > 255:
            raise ValueError("Invalid value for `reference_number`, length must be less than or equal to `255`")

        self._reference_number = reference_number
    
    @property
    def rejection_status(self):
        """Gets the rejection_status of this InvoiceReconciliationRecord.

            

        :return: The rejection_status of this InvoiceReconciliationRecord.
        :rtype: InvoiceReconciliationRecordRejectionStatus
        """
        return self._rejection_status

    @rejection_status.setter
    def rejection_status(self, rejection_status):
        """Sets the rejection_status of this InvoiceReconciliationRecord.

            

        :param rejection_status: The rejection_status of this InvoiceReconciliationRecord.
        :type: InvoiceReconciliationRecordRejectionStatus
        """

        self._rejection_status = rejection_status
    
    @property
    def resolved_by(self):
        """Gets the resolved_by of this InvoiceReconciliationRecord.

            

        :return: The resolved_by of this InvoiceReconciliationRecord.
        :rtype: int
        """
        return self._resolved_by

    @resolved_by.setter
    def resolved_by(self, resolved_by):
        """Sets the resolved_by of this InvoiceReconciliationRecord.

            

        :param resolved_by: The resolved_by of this InvoiceReconciliationRecord.
        :type: int
        """

        self._resolved_by = resolved_by
    
    @property
    def resolved_on(self):
        """Gets the resolved_on of this InvoiceReconciliationRecord.

            The resolved on date indicates when the bank transaction has been resolved.

        :return: The resolved_on of this InvoiceReconciliationRecord.
        :rtype: datetime
        """
        return self._resolved_on

    @resolved_on.setter
    def resolved_on(self, resolved_on):
        """Sets the resolved_on of this InvoiceReconciliationRecord.

            The resolved on date indicates when the bank transaction has been resolved.

        :param resolved_on: The resolved_on of this InvoiceReconciliationRecord.
        :type: datetime
        """

        self._resolved_on = resolved_on
    
    @property
    def sender_bank_account(self):
        """Gets the sender_bank_account of this InvoiceReconciliationRecord.

            

        :return: The sender_bank_account of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._sender_bank_account

    @sender_bank_account.setter
    def sender_bank_account(self, sender_bank_account):
        """Sets the sender_bank_account of this InvoiceReconciliationRecord.

            

        :param sender_bank_account: The sender_bank_account of this InvoiceReconciliationRecord.
        :type: str
        """

        self._sender_bank_account = sender_bank_account
    
    @property
    def state(self):
        """Gets the state of this InvoiceReconciliationRecord.

            The object's current state.

        :return: The state of this InvoiceReconciliationRecord.
        :rtype: InvoiceReconciliationRecordState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this InvoiceReconciliationRecord.

            The object's current state.

        :param state: The state of this InvoiceReconciliationRecord.
        :type: InvoiceReconciliationRecordState
        """

        self._state = state
    
    @property
    def street(self):
        """Gets the street of this InvoiceReconciliationRecord.

            

        :return: The street of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._street

    @street.setter
    def street(self, street):
        """Sets the street of this InvoiceReconciliationRecord.

            

        :param street: The street of this InvoiceReconciliationRecord.
        :type: str
        """

        self._street = street
    
    @property
    def type(self):
        """Gets the type of this InvoiceReconciliationRecord.

            

        :return: The type of this InvoiceReconciliationRecord.
        :rtype: InvoiceReconciliationRecordType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this InvoiceReconciliationRecord.

            

        :param type: The type of this InvoiceReconciliationRecord.
        :type: InvoiceReconciliationRecordType
        """

        self._type = type
    
    @property
    def unique_id(self):
        """Gets the unique_id of this InvoiceReconciliationRecord.

            

        :return: The unique_id of this InvoiceReconciliationRecord.
        :rtype: str
        """
        return self._unique_id

    @unique_id.setter
    def unique_id(self, unique_id):
        """Sets the unique_id of this InvoiceReconciliationRecord.

            

        :param unique_id: The unique_id of this InvoiceReconciliationRecord.
        :type: str
        """
        if unique_id is not None and len(unique_id) > 500:
            raise ValueError("Invalid value for `unique_id`, length must be less than or equal to `500`")

        self._unique_id = unique_id
    
    @property
    def value_date(self):
        """Gets the value_date of this InvoiceReconciliationRecord.

            

        :return: The value_date of this InvoiceReconciliationRecord.
        :rtype: datetime
        """
        return self._value_date

    @value_date.setter
    def value_date(self, value_date):
        """Sets the value_date of this InvoiceReconciliationRecord.

            

        :param value_date: The value_date of this InvoiceReconciliationRecord.
        :type: datetime
        """

        self._value_date = value_date
    
    @property
    def version(self):
        """Gets the version of this InvoiceReconciliationRecord.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this InvoiceReconciliationRecord.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this InvoiceReconciliationRecord.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this InvoiceReconciliationRecord.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(InvoiceReconciliationRecord, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, InvoiceReconciliationRecord):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class InstallmentCalculatedSlice:

    swagger_types = {
    
        'amount_including_tax': 'float',
        'due_on': 'datetime',
        'line_items': 'list[LineItem]',
    }

    attribute_map = {
        'amount_including_tax': 'amountIncludingTax','due_on': 'dueOn','line_items': 'lineItems',
    }

    
    _amount_including_tax = None
    _due_on = None
    _line_items = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.amount_including_tax = kwargs.get('amount_including_tax', None)
        self.due_on = kwargs.get('due_on', None)
        self.line_items = kwargs.get('line_items', None)
        

    
    @property
    def amount_including_tax(self):
        """Gets the amount_including_tax of this InstallmentCalculatedSlice.

            

        :return: The amount_including_tax of this InstallmentCalculatedSlice.
        :rtype: float
        """
        return self._amount_including_tax

    @amount_including_tax.setter
    def amount_including_tax(self, amount_including_tax):
        """Sets the amount_including_tax of this InstallmentCalculatedSlice.

            

        :param amount_including_tax: The amount_including_tax of this InstallmentCalculatedSlice.
        :type: float
        """

        self._amount_including_tax = amount_including_tax
    
    @property
    def due_on(self):
        """Gets the due_on of this InstallmentCalculatedSlice.

            

        :return: The due_on of this InstallmentCalculatedSlice.
        :rtype: datetime
        """
        return self._due_on

    @due_on.setter
    def due_on(self, due_on):
        """Sets the due_on of this InstallmentCalculatedSlice.

            

        :param due_on: The due_on of this InstallmentCalculatedSlice.
        :type: datetime
        """

        self._due_on = due_on
    
    @property
    def line_items(self):
        """Gets the line_items of this InstallmentCalculatedSlice.

            

        :return: The line_items of this InstallmentCalculatedSlice.
        :rtype: list[LineItem]
        """
        return self._line_items

    @line_items.setter
    def line_items(self, line_items):
        """Sets the line_items of this InstallmentCalculatedSlice.

            

        :param line_items: The line_items of this InstallmentCalculatedSlice.
        :type: list[LineItem]
        """

        self._line_items = line_items
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(InstallmentCalculatedSlice, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, InstallmentCalculatedSlice):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



import unittest

from constants import (
    API_CONFIG,
    SPACE_ID,
    get_transaction_create,
    FAKE_CARD_DATA,
    TEST_CARD_PAYMENT_METHOD_CONFIGURATION_ID,
    TEST_ISR_INVOICE_PAYMENT_METHOD_CONFIGURATION_ID,
)

from wallee.api import (
    TransactionServiceApi,
    CardProcessingServiceApi,
    TransactionInvoiceServiceApi,
    TransactionCompletionServiceApi,
)
from wallee.models import (
    TransactionState,
    TransactionInvoiceState,
    EntityQuery,
    EntityQueryFilter,
    EntityQueryFilterType,
    CriteriaOperator,
    TokenizationMode,
    TransactionCompletionBehavior,
    CustomersPresence,
)


class TransactionInvoiceServiceTest(unittest.TestCase):
    """TransactionInvoiceServiceApi tests"""

    def setUp(self):
        self.transaction_service = TransactionServiceApi(API_CONFIG)
        self.transaction_invoice_service = TransactionInvoiceServiceApi(API_CONFIG)
        self.transaction_completion_service = TransactionCompletionServiceApi(
            API_CONFIG)
        self.card_processing_service = CardProcessingServiceApi(API_CONFIG)

    def tearDown(self):
        pass

    def test_search(self):
        """search() should find transaction invoice by a given criteria"""

        transaction_create = get_transaction_create()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_IMMEDIATELY
        transaction = self.transaction_service.create(space_id=SPACE_ID, transaction=transaction_create)

        transaction_processed = self.card_processing_service.process(
                    space_id=SPACE_ID,
                    transaction_id=transaction.id,
                    payment_method_configuration_id=TEST_CARD_PAYMENT_METHOD_CONFIGURATION_ID,
                    card_data=FAKE_CARD_DATA,
                )

        self.assertEqual(
                TransactionState.FULFILL,
                transaction_processed.state,
                "State must be FULFILL",
            )

        entity_query_filter = EntityQueryFilter(
            # linkedTransaction does not work here as criteria
            field_name="completion.lineItemVersion.transaction.id",
            value=transaction_processed.id,
            type=EntityQueryFilterType.LEAF,
            operator=CriteriaOperator.EQUALS,
        )
        entity_query = EntityQuery(filter=entity_query_filter)

        invoices_found = self.transaction_invoice_service.search(
            space_id=SPACE_ID, query=entity_query)
        
        self.assertTrue(len(invoices_found) > 0,
                        "Should find invoice", )
        
        for invoice in invoices_found:
            self.assertEqual(TransactionInvoiceState.NOT_APPLICABLE, invoice.state,
                             "Invoice paid by card is expected to be of NOT_APPLICABLE state")



    def test_derecognize_transaction_invoice(self):
        """mark_as_derecognized() should derecognize open invoice"""

        transaction_create = get_transaction_create()
        transaction_create.tokenization_mode = TokenizationMode.FORCE_CREATION
        transaction_create.customers_presence = CustomersPresence.NOT_PRESENT
        transaction_create.completion_behavior = TransactionCompletionBehavior.COMPLETE_DEFERRED

        # we want invoice in OPEN state (OPEN invoices can be derecognized), so we force payment by invoice
        transaction_create.allowed_payment_method_configurations = [TEST_ISR_INVOICE_PAYMENT_METHOD_CONFIGURATION_ID]
        transaction = self.transaction_service.create(space_id=SPACE_ID, transaction=transaction_create)

        transaction_processed = self.transaction_service.process_without_user_interaction(
            space_id=SPACE_ID, id=transaction.id)
        
        transaction_completion = self.transaction_completion_service.complete_online(
                    space_id=SPACE_ID, id=transaction_processed.id)
        
        entity_query_filter = EntityQueryFilter(
            field_name="completion.lineItemVersion.transaction.id",
            value=transaction_processed.id,
            type=EntityQueryFilterType.LEAF,
            operator=CriteriaOperator.EQUALS,
        )
        entity_query = EntityQuery(filter=entity_query_filter)

        invoices_found = self.transaction_invoice_service.search(
            space_id=SPACE_ID, query=entity_query)
        
        self.assertTrue(len(invoices_found) > 0,
                        "Should find invoice", )
        
        found_invoice = invoices_found[0]

        self.assertEqual(TransactionInvoiceState.OPEN, found_invoice.state,
                         "Transaction paid by invoice should create invoice in OPEN state" )
        
        derecognized_invoice = self.transaction_invoice_service.mark_as_derecognized(
            space_id=SPACE_ID, id=found_invoice.id);
        
        self.assertEqual(TransactionInvoiceState.DERECOGNIZED, derecognized_invoice.state,
                         "Expected DERECOGNIZED invoice state")


if __name__ == "__main__":
    unittest.main(failfast=True)



# coding: utf-8
from enum import Enum, unique


@unique
class PaymentTerminalConfigurationState(Enum):
    
    CREATE = "CREATE"
    ACTIVE = "ACTIVE"
    DELETING = "DELETING"
    DELETED = "DELETED"




# coding: utf-8
import pprint
import six
from enum import Enum



class ProductSetupFee:

    swagger_types = {
    
        'component': 'SubscriptionProductComponent',
        'description': 'dict(str, str)',
        'id': 'int',
        'linked_space_id': 'int',
        'name': 'dict(str, str)',
        'on_downgrade_credited_amount': 'list[PersistableCurrencyAmount]',
        'on_upgrade_credited_amount': 'list[PersistableCurrencyAmount]',
        'setup_fee': 'list[PersistableCurrencyAmount]',
        'type': 'ProductFeeType',
        'version': 'int',
    }

    attribute_map = {
        'component': 'component','description': 'description','id': 'id','linked_space_id': 'linkedSpaceId','name': 'name','on_downgrade_credited_amount': 'onDowngradeCreditedAmount','on_upgrade_credited_amount': 'onUpgradeCreditedAmount','setup_fee': 'setupFee','type': 'type','version': 'version',
    }

    
    _component = None
    _description = None
    _id = None
    _linked_space_id = None
    _name = None
    _on_downgrade_credited_amount = None
    _on_upgrade_credited_amount = None
    _setup_fee = None
    _type = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.component = kwargs.get('component', None)
        self.description = kwargs.get('description', None)
        self.id = kwargs.get('id', None)
        self.linked_space_id = kwargs.get('linked_space_id', None)
        self.name = kwargs.get('name', None)
        self.on_downgrade_credited_amount = kwargs.get('on_downgrade_credited_amount', None)
        self.on_upgrade_credited_amount = kwargs.get('on_upgrade_credited_amount', None)
        self.setup_fee = kwargs.get('setup_fee', None)
        self.type = kwargs.get('type', None)
        self.version = kwargs.get('version', None)
        

    
    @property
    def component(self):
        """Gets the component of this ProductSetupFee.

            

        :return: The component of this ProductSetupFee.
        :rtype: SubscriptionProductComponent
        """
        return self._component

    @component.setter
    def component(self, component):
        """Sets the component of this ProductSetupFee.

            

        :param component: The component of this ProductSetupFee.
        :type: SubscriptionProductComponent
        """

        self._component = component
    
    @property
    def description(self):
        """Gets the description of this ProductSetupFee.

            The description of a component fee describes the fee to the subscriber. The description may be shown in documents or on certain user interfaces.

        :return: The description of this ProductSetupFee.
        :rtype: dict(str, str)
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this ProductSetupFee.

            The description of a component fee describes the fee to the subscriber. The description may be shown in documents or on certain user interfaces.

        :param description: The description of this ProductSetupFee.
        :type: dict(str, str)
        """

        self._description = description
    
    @property
    def id(self):
        """Gets the id of this ProductSetupFee.

            A unique identifier for the object.

        :return: The id of this ProductSetupFee.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ProductSetupFee.

            A unique identifier for the object.

        :param id: The id of this ProductSetupFee.
        :type: int
        """

        self._id = id
    
    @property
    def linked_space_id(self):
        """Gets the linked_space_id of this ProductSetupFee.

            The ID of the space this object belongs to.

        :return: The linked_space_id of this ProductSetupFee.
        :rtype: int
        """
        return self._linked_space_id

    @linked_space_id.setter
    def linked_space_id(self, linked_space_id):
        """Sets the linked_space_id of this ProductSetupFee.

            The ID of the space this object belongs to.

        :param linked_space_id: The linked_space_id of this ProductSetupFee.
        :type: int
        """

        self._linked_space_id = linked_space_id
    
    @property
    def name(self):
        """Gets the name of this ProductSetupFee.

            The name of the fee should describe for the subscriber in few words for what the fee is for.

        :return: The name of this ProductSetupFee.
        :rtype: dict(str, str)
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ProductSetupFee.

            The name of the fee should describe for the subscriber in few words for what the fee is for.

        :param name: The name of this ProductSetupFee.
        :type: dict(str, str)
        """

        self._name = name
    
    @property
    def on_downgrade_credited_amount(self):
        """Gets the on_downgrade_credited_amount of this ProductSetupFee.

            When the subscription is changed and the change is considered as a downgrade the amount defined by this property will be credited to the subscriber.

        :return: The on_downgrade_credited_amount of this ProductSetupFee.
        :rtype: list[PersistableCurrencyAmount]
        """
        return self._on_downgrade_credited_amount

    @on_downgrade_credited_amount.setter
    def on_downgrade_credited_amount(self, on_downgrade_credited_amount):
        """Sets the on_downgrade_credited_amount of this ProductSetupFee.

            When the subscription is changed and the change is considered as a downgrade the amount defined by this property will be credited to the subscriber.

        :param on_downgrade_credited_amount: The on_downgrade_credited_amount of this ProductSetupFee.
        :type: list[PersistableCurrencyAmount]
        """

        self._on_downgrade_credited_amount = on_downgrade_credited_amount
    
    @property
    def on_upgrade_credited_amount(self):
        """Gets the on_upgrade_credited_amount of this ProductSetupFee.

            When the subscription is changed and the change is considered as a upgrade the amount defined by this property will be credited to the subscriber.

        :return: The on_upgrade_credited_amount of this ProductSetupFee.
        :rtype: list[PersistableCurrencyAmount]
        """
        return self._on_upgrade_credited_amount

    @on_upgrade_credited_amount.setter
    def on_upgrade_credited_amount(self, on_upgrade_credited_amount):
        """Sets the on_upgrade_credited_amount of this ProductSetupFee.

            When the subscription is changed and the change is considered as a upgrade the amount defined by this property will be credited to the subscriber.

        :param on_upgrade_credited_amount: The on_upgrade_credited_amount of this ProductSetupFee.
        :type: list[PersistableCurrencyAmount]
        """

        self._on_upgrade_credited_amount = on_upgrade_credited_amount
    
    @property
    def setup_fee(self):
        """Gets the setup_fee of this ProductSetupFee.

            The setup fee is charged when the subscriber subscribes to this component. The setup fee is debited with the first charge for the subscriptions.

        :return: The setup_fee of this ProductSetupFee.
        :rtype: list[PersistableCurrencyAmount]
        """
        return self._setup_fee

    @setup_fee.setter
    def setup_fee(self, setup_fee):
        """Sets the setup_fee of this ProductSetupFee.

            The setup fee is charged when the subscriber subscribes to this component. The setup fee is debited with the first charge for the subscriptions.

        :param setup_fee: The setup_fee of this ProductSetupFee.
        :type: list[PersistableCurrencyAmount]
        """

        self._setup_fee = setup_fee
    
    @property
    def type(self):
        """Gets the type of this ProductSetupFee.

            

        :return: The type of this ProductSetupFee.
        :rtype: ProductFeeType
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ProductSetupFee.

            

        :param type: The type of this ProductSetupFee.
        :type: ProductFeeType
        """

        self._type = type
    
    @property
    def version(self):
        """Gets the version of this ProductSetupFee.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this ProductSetupFee.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ProductSetupFee.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this ProductSetupFee.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(ProductSetupFee, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, ProductSetupFee):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class EntityQueryOrderBy:

    swagger_types = {
    
        'field_name': 'str',
        'sorting': 'EntityQueryOrderByType',
    }

    attribute_map = {
        'field_name': 'fieldName','sorting': 'sorting',
    }

    
    _field_name = None
    _sorting = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.field_name = kwargs.get('field_name')

        self.sorting = kwargs.get('sorting')

        

    
    @property
    def field_name(self):
        """Gets the field_name of this EntityQueryOrderBy.

            

        :return: The field_name of this EntityQueryOrderBy.
        :rtype: str
        """
        return self._field_name

    @field_name.setter
    def field_name(self, field_name):
        """Sets the field_name of this EntityQueryOrderBy.

            

        :param field_name: The field_name of this EntityQueryOrderBy.
        :type: str
        """
        if field_name is None:
            raise ValueError("Invalid value for `field_name`, must not be `None`")

        self._field_name = field_name
    
    @property
    def sorting(self):
        """Gets the sorting of this EntityQueryOrderBy.

            

        :return: The sorting of this EntityQueryOrderBy.
        :rtype: EntityQueryOrderByType
        """
        return self._sorting

    @sorting.setter
    def sorting(self, sorting):
        """Sets the sorting of this EntityQueryOrderBy.

            

        :param sorting: The sorting of this EntityQueryOrderBy.
        :type: EntityQueryOrderByType
        """
        if sorting is None:
            raise ValueError("Invalid value for `sorting`, must not be `None`")

        self._sorting = sorting
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(EntityQueryOrderBy, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, EntityQueryOrderBy):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
from enum import Enum, unique


@unique
class PaymentTerminalLocationVersionState(Enum):
    
    PENDING = "PENDING"
    SCHEDULING = "SCHEDULING"
    ACTIVE = "ACTIVE"
    DELETING = "DELETING"
    DELETED = "DELETED"




# coding: utf-8
import pprint
import six
from enum import Enum



class TransactionCompletionRequest:

    swagger_types = {
    
        'external_id': 'str',
        'invoice_merchant_reference': 'str',
        'last_completion': 'bool',
        'line_items': 'list[CompletionLineItemCreate]',
        'statement_descriptor': 'str',
        'transaction_id': 'int',
    }

    attribute_map = {
        'external_id': 'externalId','invoice_merchant_reference': 'invoiceMerchantReference','last_completion': 'lastCompletion','line_items': 'lineItems','statement_descriptor': 'statementDescriptor','transaction_id': 'transactionId',
    }

    
    _external_id = None
    _invoice_merchant_reference = None
    _last_completion = None
    _line_items = None
    _statement_descriptor = None
    _transaction_id = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.external_id = kwargs.get('external_id')

        self.invoice_merchant_reference = kwargs.get('invoice_merchant_reference', None)
        self.last_completion = kwargs.get('last_completion')

        self.line_items = kwargs.get('line_items', None)
        self.statement_descriptor = kwargs.get('statement_descriptor', None)
        self.transaction_id = kwargs.get('transaction_id')

        

    
    @property
    def external_id(self):
        """Gets the external_id of this TransactionCompletionRequest.

            The external ID helps to identify the entity and a subsequent creation of an entity with the same ID will not create a new entity.

        :return: The external_id of this TransactionCompletionRequest.
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this TransactionCompletionRequest.

            The external ID helps to identify the entity and a subsequent creation of an entity with the same ID will not create a new entity.

        :param external_id: The external_id of this TransactionCompletionRequest.
        :type: str
        """
        if external_id is None:
            raise ValueError("Invalid value for `external_id`, must not be `None`")
        if external_id is not None and len(external_id) > 100:
            raise ValueError("Invalid value for `external_id`, length must be less than or equal to `100`")
        if external_id is not None and len(external_id) < 1:
            raise ValueError("Invalid value for `external_id`, length must be greater than or equal to `1`")

        self._external_id = external_id
    
    @property
    def invoice_merchant_reference(self):
        """Gets the invoice_merchant_reference of this TransactionCompletionRequest.

            

        :return: The invoice_merchant_reference of this TransactionCompletionRequest.
        :rtype: str
        """
        return self._invoice_merchant_reference

    @invoice_merchant_reference.setter
    def invoice_merchant_reference(self, invoice_merchant_reference):
        """Sets the invoice_merchant_reference of this TransactionCompletionRequest.

            

        :param invoice_merchant_reference: The invoice_merchant_reference of this TransactionCompletionRequest.
        :type: str
        """
        if invoice_merchant_reference is not None and len(invoice_merchant_reference) > 100:
            raise ValueError("Invalid value for `invoice_merchant_reference`, length must be less than or equal to `100`")

        self._invoice_merchant_reference = invoice_merchant_reference
    
    @property
    def last_completion(self):
        """Gets the last_completion of this TransactionCompletionRequest.

            The last completion flag indicates if this is the last completion. After the last completion is created no further completions can be issued.

        :return: The last_completion of this TransactionCompletionRequest.
        :rtype: bool
        """
        return self._last_completion

    @last_completion.setter
    def last_completion(self, last_completion):
        """Sets the last_completion of this TransactionCompletionRequest.

            The last completion flag indicates if this is the last completion. After the last completion is created no further completions can be issued.

        :param last_completion: The last_completion of this TransactionCompletionRequest.
        :type: bool
        """
        if last_completion is None:
            raise ValueError("Invalid value for `last_completion`, must not be `None`")

        self._last_completion = last_completion
    
    @property
    def line_items(self):
        """Gets the line_items of this TransactionCompletionRequest.

            The line items which will be used to complete the transaction.

        :return: The line_items of this TransactionCompletionRequest.
        :rtype: list[CompletionLineItemCreate]
        """
        return self._line_items

    @line_items.setter
    def line_items(self, line_items):
        """Sets the line_items of this TransactionCompletionRequest.

            The line items which will be used to complete the transaction.

        :param line_items: The line_items of this TransactionCompletionRequest.
        :type: list[CompletionLineItemCreate]
        """

        self._line_items = line_items
    
    @property
    def statement_descriptor(self):
        """Gets the statement_descriptor of this TransactionCompletionRequest.

            The statement descriptor explain charges or payments on bank statements.

        :return: The statement_descriptor of this TransactionCompletionRequest.
        :rtype: str
        """
        return self._statement_descriptor

    @statement_descriptor.setter
    def statement_descriptor(self, statement_descriptor):
        """Sets the statement_descriptor of this TransactionCompletionRequest.

            The statement descriptor explain charges or payments on bank statements.

        :param statement_descriptor: The statement_descriptor of this TransactionCompletionRequest.
        :type: str
        """
        if statement_descriptor is not None and len(statement_descriptor) > 80:
            raise ValueError("Invalid value for `statement_descriptor`, length must be less than or equal to `80`")

        self._statement_descriptor = statement_descriptor
    
    @property
    def transaction_id(self):
        """Gets the transaction_id of this TransactionCompletionRequest.

            The ID of the transaction which should be completed.

        :return: The transaction_id of this TransactionCompletionRequest.
        :rtype: int
        """
        return self._transaction_id

    @transaction_id.setter
    def transaction_id(self, transaction_id):
        """Sets the transaction_id of this TransactionCompletionRequest.

            The ID of the transaction which should be completed.

        :param transaction_id: The transaction_id of this TransactionCompletionRequest.
        :type: int
        """
        if transaction_id is None:
            raise ValueError("Invalid value for `transaction_id`, must not be `None`")

        self._transaction_id = transaction_id
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(TransactionCompletionRequest, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, TransactionCompletionRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum
from . import AbstractSubscriberUpdate


class SubscriberCreate(AbstractSubscriberUpdate):

    swagger_types = {
    
        'state': 'CreationEntityState',
        'external_id': 'str',
    }

    attribute_map = {
        'state': 'state','external_id': 'externalId',
    }

    
    _state = None
    _external_id = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.state = kwargs.get('state', None)
        self.external_id = kwargs.get('external_id')

        super().__init__(**kwargs)
        self.swagger_types.update(super().swagger_types)
        self.attribute_map.update(super().attribute_map)

    
    @property
    def state(self):
        """Gets the state of this SubscriberCreate.

            The object's current state.

        :return: The state of this SubscriberCreate.
        :rtype: CreationEntityState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this SubscriberCreate.

            The object's current state.

        :param state: The state of this SubscriberCreate.
        :type: CreationEntityState
        """

        self._state = state
    
    @property
    def external_id(self):
        """Gets the external_id of this SubscriberCreate.

            A client generated nonce which identifies the entity to be created. Subsequent creation requests with the same external ID will not create new entities but return the initially created entity instead.

        :return: The external_id of this SubscriberCreate.
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this SubscriberCreate.

            A client generated nonce which identifies the entity to be created. Subsequent creation requests with the same external ID will not create new entities but return the initially created entity instead.

        :param external_id: The external_id of this SubscriberCreate.
        :type: str
        """
        if external_id is None:
            raise ValueError("Invalid value for `external_id`, must not be `None`")

        self._external_id = external_id
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(SubscriberCreate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, SubscriberCreate):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class MetricUsage:

    swagger_types = {
    
        'consumed_units': 'float',
        'metric_description': 'dict(str, str)',
        'metric_id': 'int',
        'metric_name': 'dict(str, str)',
    }

    attribute_map = {
        'consumed_units': 'consumedUnits','metric_description': 'metricDescription','metric_id': 'metricId','metric_name': 'metricName',
    }

    
    _consumed_units = None
    _metric_description = None
    _metric_id = None
    _metric_name = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.consumed_units = kwargs.get('consumed_units', None)
        self.metric_description = kwargs.get('metric_description', None)
        self.metric_id = kwargs.get('metric_id', None)
        self.metric_name = kwargs.get('metric_name', None)
        

    
    @property
    def consumed_units(self):
        """Gets the consumed_units of this MetricUsage.

            The consumed units provide the value of how much has been consumed of the particular metric.

        :return: The consumed_units of this MetricUsage.
        :rtype: float
        """
        return self._consumed_units

    @consumed_units.setter
    def consumed_units(self, consumed_units):
        """Sets the consumed_units of this MetricUsage.

            The consumed units provide the value of how much has been consumed of the particular metric.

        :param consumed_units: The consumed_units of this MetricUsage.
        :type: float
        """

        self._consumed_units = consumed_units
    
    @property
    def metric_description(self):
        """Gets the metric_description of this MetricUsage.

            The metric description describes the metric.

        :return: The metric_description of this MetricUsage.
        :rtype: dict(str, str)
        """
        return self._metric_description

    @metric_description.setter
    def metric_description(self, metric_description):
        """Sets the metric_description of this MetricUsage.

            The metric description describes the metric.

        :param metric_description: The metric_description of this MetricUsage.
        :type: dict(str, str)
        """

        self._metric_description = metric_description
    
    @property
    def metric_id(self):
        """Gets the metric_id of this MetricUsage.

            The metric ID identifies the metric for consumed units.

        :return: The metric_id of this MetricUsage.
        :rtype: int
        """
        return self._metric_id

    @metric_id.setter
    def metric_id(self, metric_id):
        """Sets the metric_id of this MetricUsage.

            The metric ID identifies the metric for consumed units.

        :param metric_id: The metric_id of this MetricUsage.
        :type: int
        """

        self._metric_id = metric_id
    
    @property
    def metric_name(self):
        """Gets the metric_name of this MetricUsage.

            The metric name defines the name of the consumed units.

        :return: The metric_name of this MetricUsage.
        :rtype: dict(str, str)
        """
        return self._metric_name

    @metric_name.setter
    def metric_name(self, metric_name):
        """Sets the metric_name of this MetricUsage.

            The metric name defines the name of the consumed units.

        :param metric_name: The metric_name of this MetricUsage.
        :type: dict(str, str)
        """

        self._metric_name = metric_name
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(MetricUsage, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, MetricUsage):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class ShopifyTaxLine:

    swagger_types = {
    
        'fraction_rate': 'float',
        'id': 'int',
        'rate': 'float',
        'title': 'str',
        'version': 'int',
    }

    attribute_map = {
        'fraction_rate': 'fractionRate','id': 'id','rate': 'rate','title': 'title','version': 'version',
    }

    
    _fraction_rate = None
    _id = None
    _rate = None
    _title = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.fraction_rate = kwargs.get('fraction_rate', None)
        self.id = kwargs.get('id', None)
        self.rate = kwargs.get('rate', None)
        self.title = kwargs.get('title', None)
        self.version = kwargs.get('version', None)
        

    
    @property
    def fraction_rate(self):
        """Gets the fraction_rate of this ShopifyTaxLine.

            

        :return: The fraction_rate of this ShopifyTaxLine.
        :rtype: float
        """
        return self._fraction_rate

    @fraction_rate.setter
    def fraction_rate(self, fraction_rate):
        """Sets the fraction_rate of this ShopifyTaxLine.

            

        :param fraction_rate: The fraction_rate of this ShopifyTaxLine.
        :type: float
        """

        self._fraction_rate = fraction_rate
    
    @property
    def id(self):
        """Gets the id of this ShopifyTaxLine.

            A unique identifier for the object.

        :return: The id of this ShopifyTaxLine.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ShopifyTaxLine.

            A unique identifier for the object.

        :param id: The id of this ShopifyTaxLine.
        :type: int
        """

        self._id = id
    
    @property
    def rate(self):
        """Gets the rate of this ShopifyTaxLine.

            

        :return: The rate of this ShopifyTaxLine.
        :rtype: float
        """
        return self._rate

    @rate.setter
    def rate(self, rate):
        """Sets the rate of this ShopifyTaxLine.

            

        :param rate: The rate of this ShopifyTaxLine.
        :type: float
        """

        self._rate = rate
    
    @property
    def title(self):
        """Gets the title of this ShopifyTaxLine.

            

        :return: The title of this ShopifyTaxLine.
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this ShopifyTaxLine.

            

        :param title: The title of this ShopifyTaxLine.
        :type: str
        """

        self._title = title
    
    @property
    def version(self):
        """Gets the version of this ShopifyTaxLine.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this ShopifyTaxLine.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ShopifyTaxLine.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this ShopifyTaxLine.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(ShopifyTaxLine, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, ShopifyTaxLine):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8

from __future__ import absolute_import

import six
import re

from wallee.api_client import ApiClient
from wallee.encryption_util import EncryptionUtil

class WebhookEncryptionServiceApi:

    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)

    def read(self, id, **kwargs):
        """Read

        Reads the entity with the given 'id' and returns it.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True.
        
        >>> thread = api.read(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: The ID of the key version. (required)
        :return: WebhookEncryptionPublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        
        kwargs['request_timeout'] = self.api_client.configuration.request_timeout
        if kwargs.get('async_req'):
            return self.read_with_http_info(id, **kwargs)
        else:
            (data) = self.read_with_http_info(id, **kwargs)
            return data

    def read_with_http_info(self, id, **kwargs):
        """Read

        Reads the entity with the given 'id' and returns it.
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True.
        
        >>> thread = api.read_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: The ID of the key version. (required)
        :return: WebhookEncryptionPublicKey
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method read" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `read`")

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'id' in params:
            query_params.append(('id', params['id']))

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json;charset=utf-8'])

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(
            ['*/*'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(
            '/webhook-encryption/read', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='WebhookEncryptionPublicKey',
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('request_timeout'),
            collection_formats=collection_formats)

    def is_content_valid(self, signature_header, content_to_verify):
        """Verify webhook content using signature header

        Verifies webhook content using signature header.

        :param str signature_header: Signature header 'X-Signature' value from the Http request
        :param str content_to_verify: Raw webhook content in String format
        :return: bool if webhook content body conforms with signature header
        """
        regex = r"^algorithm=([a-zA-Z0-9]+),\skeyId=([a-z0-9\-]+),\s{1}signature=([a-zA-Z0-9+\/=]+)$"
        pattern = re.compile(regex)
        matcher = pattern.match(signature_header)

        if matcher:
            signature_algorithm = matcher.group(1)
            public_key_id = matcher.group(2)
            content_signature = matcher.group(3)

            public_key_response = WebhookEncryptionServiceApi.read(self, public_key_id)

            if public_key_response is None or public_key_response.public_key is None:
                raise ValueError(f"Could not find public key with id {public_key_id}")

            return EncryptionUtil.is_content_valid(content_to_verify, content_signature, public_key_response.public_key,
                                                   signature_algorithm)
        else:
            raise ValueError(
                "Invalid webhook signature header. Expected format: 'algorithm=<algorithm>, keyId=<keyId>, "
                "signature=<signature>'")



# coding: utf-8
import pprint
import six
from enum import Enum



class SubscriptionPeriodBill:

    swagger_types = {
    
        'created_on': 'datetime',
        'effective_period_end_date': 'datetime',
        'id': 'int',
        'language': 'str',
        'linked_space_id': 'int',
        'period_start_date': 'datetime',
        'planned_period_end_date': 'datetime',
        'planned_purge_date': 'datetime',
        'state': 'SubscriptionPeriodBillState',
        'subscription_version': 'SubscriptionVersion',
        'version': 'int',
    }

    attribute_map = {
        'created_on': 'createdOn','effective_period_end_date': 'effectivePeriodEndDate','id': 'id','language': 'language','linked_space_id': 'linkedSpaceId','period_start_date': 'periodStartDate','planned_period_end_date': 'plannedPeriodEndDate','planned_purge_date': 'plannedPurgeDate','state': 'state','subscription_version': 'subscriptionVersion','version': 'version',
    }

    
    _created_on = None
    _effective_period_end_date = None
    _id = None
    _language = None
    _linked_space_id = None
    _period_start_date = None
    _planned_period_end_date = None
    _planned_purge_date = None
    _state = None
    _subscription_version = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.created_on = kwargs.get('created_on', None)
        self.effective_period_end_date = kwargs.get('effective_period_end_date', None)
        self.id = kwargs.get('id', None)
        self.language = kwargs.get('language', None)
        self.linked_space_id = kwargs.get('linked_space_id', None)
        self.period_start_date = kwargs.get('period_start_date', None)
        self.planned_period_end_date = kwargs.get('planned_period_end_date', None)
        self.planned_purge_date = kwargs.get('planned_purge_date', None)
        self.state = kwargs.get('state', None)
        self.subscription_version = kwargs.get('subscription_version', None)
        self.version = kwargs.get('version', None)
        

    
    @property
    def created_on(self):
        """Gets the created_on of this SubscriptionPeriodBill.

            

        :return: The created_on of this SubscriptionPeriodBill.
        :rtype: datetime
        """
        return self._created_on

    @created_on.setter
    def created_on(self, created_on):
        """Sets the created_on of this SubscriptionPeriodBill.

            

        :param created_on: The created_on of this SubscriptionPeriodBill.
        :type: datetime
        """

        self._created_on = created_on
    
    @property
    def effective_period_end_date(self):
        """Gets the effective_period_end_date of this SubscriptionPeriodBill.

            

        :return: The effective_period_end_date of this SubscriptionPeriodBill.
        :rtype: datetime
        """
        return self._effective_period_end_date

    @effective_period_end_date.setter
    def effective_period_end_date(self, effective_period_end_date):
        """Sets the effective_period_end_date of this SubscriptionPeriodBill.

            

        :param effective_period_end_date: The effective_period_end_date of this SubscriptionPeriodBill.
        :type: datetime
        """

        self._effective_period_end_date = effective_period_end_date
    
    @property
    def id(self):
        """Gets the id of this SubscriptionPeriodBill.

            A unique identifier for the object.

        :return: The id of this SubscriptionPeriodBill.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SubscriptionPeriodBill.

            A unique identifier for the object.

        :param id: The id of this SubscriptionPeriodBill.
        :type: int
        """

        self._id = id
    
    @property
    def language(self):
        """Gets the language of this SubscriptionPeriodBill.

            The language that is linked to the object.

        :return: The language of this SubscriptionPeriodBill.
        :rtype: str
        """
        return self._language

    @language.setter
    def language(self, language):
        """Sets the language of this SubscriptionPeriodBill.

            The language that is linked to the object.

        :param language: The language of this SubscriptionPeriodBill.
        :type: str
        """

        self._language = language
    
    @property
    def linked_space_id(self):
        """Gets the linked_space_id of this SubscriptionPeriodBill.

            The ID of the space this object belongs to.

        :return: The linked_space_id of this SubscriptionPeriodBill.
        :rtype: int
        """
        return self._linked_space_id

    @linked_space_id.setter
    def linked_space_id(self, linked_space_id):
        """Sets the linked_space_id of this SubscriptionPeriodBill.

            The ID of the space this object belongs to.

        :param linked_space_id: The linked_space_id of this SubscriptionPeriodBill.
        :type: int
        """

        self._linked_space_id = linked_space_id
    
    @property
    def period_start_date(self):
        """Gets the period_start_date of this SubscriptionPeriodBill.

            

        :return: The period_start_date of this SubscriptionPeriodBill.
        :rtype: datetime
        """
        return self._period_start_date

    @period_start_date.setter
    def period_start_date(self, period_start_date):
        """Sets the period_start_date of this SubscriptionPeriodBill.

            

        :param period_start_date: The period_start_date of this SubscriptionPeriodBill.
        :type: datetime
        """

        self._period_start_date = period_start_date
    
    @property
    def planned_period_end_date(self):
        """Gets the planned_period_end_date of this SubscriptionPeriodBill.

            

        :return: The planned_period_end_date of this SubscriptionPeriodBill.
        :rtype: datetime
        """
        return self._planned_period_end_date

    @planned_period_end_date.setter
    def planned_period_end_date(self, planned_period_end_date):
        """Sets the planned_period_end_date of this SubscriptionPeriodBill.

            

        :param planned_period_end_date: The planned_period_end_date of this SubscriptionPeriodBill.
        :type: datetime
        """

        self._planned_period_end_date = planned_period_end_date
    
    @property
    def planned_purge_date(self):
        """Gets the planned_purge_date of this SubscriptionPeriodBill.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :return: The planned_purge_date of this SubscriptionPeriodBill.
        :rtype: datetime
        """
        return self._planned_purge_date

    @planned_purge_date.setter
    def planned_purge_date(self, planned_purge_date):
        """Sets the planned_purge_date of this SubscriptionPeriodBill.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :param planned_purge_date: The planned_purge_date of this SubscriptionPeriodBill.
        :type: datetime
        """

        self._planned_purge_date = planned_purge_date
    
    @property
    def state(self):
        """Gets the state of this SubscriptionPeriodBill.

            The object's current state.

        :return: The state of this SubscriptionPeriodBill.
        :rtype: SubscriptionPeriodBillState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this SubscriptionPeriodBill.

            The object's current state.

        :param state: The state of this SubscriptionPeriodBill.
        :type: SubscriptionPeriodBillState
        """

        self._state = state
    
    @property
    def subscription_version(self):
        """Gets the subscription_version of this SubscriptionPeriodBill.

            

        :return: The subscription_version of this SubscriptionPeriodBill.
        :rtype: SubscriptionVersion
        """
        return self._subscription_version

    @subscription_version.setter
    def subscription_version(self, subscription_version):
        """Sets the subscription_version of this SubscriptionPeriodBill.

            

        :param subscription_version: The subscription_version of this SubscriptionPeriodBill.
        :type: SubscriptionVersion
        """

        self._subscription_version = subscription_version
    
    @property
    def version(self):
        """Gets the version of this SubscriptionPeriodBill.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this SubscriptionPeriodBill.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this SubscriptionPeriodBill.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this SubscriptionPeriodBill.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(SubscriptionPeriodBill, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, SubscriptionPeriodBill):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class SubscriptionMetricUsageReportCreate:

    swagger_types = {
    
        'consumed_units': 'float',
        'description': 'str',
        'external_id': 'str',
        'metric': 'int',
        'subscription': 'int',
    }

    attribute_map = {
        'consumed_units': 'consumedUnits','description': 'description','external_id': 'externalId','metric': 'metric','subscription': 'subscription',
    }

    
    _consumed_units = None
    _description = None
    _external_id = None
    _metric = None
    _subscription = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.consumed_units = kwargs.get('consumed_units')

        self.description = kwargs.get('description', None)
        self.external_id = kwargs.get('external_id')

        self.metric = kwargs.get('metric')

        self.subscription = kwargs.get('subscription')

        

    
    @property
    def consumed_units(self):
        """Gets the consumed_units of this SubscriptionMetricUsageReportCreate.

            The consumed units describe the amount of resources consumed. Those consumed units will be billed in the next billing cycle.

        :return: The consumed_units of this SubscriptionMetricUsageReportCreate.
        :rtype: float
        """
        return self._consumed_units

    @consumed_units.setter
    def consumed_units(self, consumed_units):
        """Sets the consumed_units of this SubscriptionMetricUsageReportCreate.

            The consumed units describe the amount of resources consumed. Those consumed units will be billed in the next billing cycle.

        :param consumed_units: The consumed_units of this SubscriptionMetricUsageReportCreate.
        :type: float
        """
        if consumed_units is None:
            raise ValueError("Invalid value for `consumed_units`, must not be `None`")

        self._consumed_units = consumed_units
    
    @property
    def description(self):
        """Gets the description of this SubscriptionMetricUsageReportCreate.

            The metric usage report description describe the reported usage. This description may be shown to the end user.

        :return: The description of this SubscriptionMetricUsageReportCreate.
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this SubscriptionMetricUsageReportCreate.

            The metric usage report description describe the reported usage. This description may be shown to the end user.

        :param description: The description of this SubscriptionMetricUsageReportCreate.
        :type: str
        """
        if description is not None and len(description) > 100:
            raise ValueError("Invalid value for `description`, length must be less than or equal to `100`")

        self._description = description
    
    @property
    def external_id(self):
        """Gets the external_id of this SubscriptionMetricUsageReportCreate.

            The external id identifies the metric usage uniquely.

        :return: The external_id of this SubscriptionMetricUsageReportCreate.
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this SubscriptionMetricUsageReportCreate.

            The external id identifies the metric usage uniquely.

        :param external_id: The external_id of this SubscriptionMetricUsageReportCreate.
        :type: str
        """
        if external_id is None:
            raise ValueError("Invalid value for `external_id`, must not be `None`")

        self._external_id = external_id
    
    @property
    def metric(self):
        """Gets the metric of this SubscriptionMetricUsageReportCreate.

            The metric usage report is linked to the metric for which the usage should be recorded.

        :return: The metric of this SubscriptionMetricUsageReportCreate.
        :rtype: int
        """
        return self._metric

    @metric.setter
    def metric(self, metric):
        """Sets the metric of this SubscriptionMetricUsageReportCreate.

            The metric usage report is linked to the metric for which the usage should be recorded.

        :param metric: The metric of this SubscriptionMetricUsageReportCreate.
        :type: int
        """
        if metric is None:
            raise ValueError("Invalid value for `metric`, must not be `None`")

        self._metric = metric
    
    @property
    def subscription(self):
        """Gets the subscription of this SubscriptionMetricUsageReportCreate.

            The subscription to which the usage is added to.

        :return: The subscription of this SubscriptionMetricUsageReportCreate.
        :rtype: int
        """
        return self._subscription

    @subscription.setter
    def subscription(self, subscription):
        """Sets the subscription of this SubscriptionMetricUsageReportCreate.

            The subscription to which the usage is added to.

        :param subscription: The subscription of this SubscriptionMetricUsageReportCreate.
        :type: int
        """
        if subscription is None:
            raise ValueError("Invalid value for `subscription`, must not be `None`")

        self._subscription = subscription
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(SubscriptionMetricUsageReportCreate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, SubscriptionMetricUsageReportCreate):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class AnalyticsSchemaColumn:

    swagger_types = {
    
        'alias_name': 'str',
        'column_name': 'str',
        'description': 'dict(str, str)',
        'precision': 'int',
        'referenced_table': 'str',
        'scale': 'int',
        'table_name': 'str',
        'type': 'str',
    }

    attribute_map = {
        'alias_name': 'aliasName','column_name': 'columnName','description': 'description','precision': 'precision','referenced_table': 'referencedTable','scale': 'scale','table_name': 'tableName','type': 'type',
    }

    
    _alias_name = None
    _column_name = None
    _description = None
    _precision = None
    _referenced_table = None
    _scale = None
    _table_name = None
    _type = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.alias_name = kwargs.get('alias_name', None)
        self.column_name = kwargs.get('column_name', None)
        self.description = kwargs.get('description', None)
        self.precision = kwargs.get('precision', None)
        self.referenced_table = kwargs.get('referenced_table', None)
        self.scale = kwargs.get('scale', None)
        self.table_name = kwargs.get('table_name', None)
        self.type = kwargs.get('type', None)
        

    
    @property
    def alias_name(self):
        """Gets the alias_name of this AnalyticsSchemaColumn.

            The name of the alias defined for the column in the query or null if none is defined.

        :return: The alias_name of this AnalyticsSchemaColumn.
        :rtype: str
        """
        return self._alias_name

    @alias_name.setter
    def alias_name(self, alias_name):
        """Sets the alias_name of this AnalyticsSchemaColumn.

            The name of the alias defined for the column in the query or null if none is defined.

        :param alias_name: The alias_name of this AnalyticsSchemaColumn.
        :type: str
        """

        self._alias_name = alias_name
    
    @property
    def column_name(self):
        """Gets the column_name of this AnalyticsSchemaColumn.

            The name of the column in the table or null if this is a synthetic column which is the result of some SQL expression.

        :return: The column_name of this AnalyticsSchemaColumn.
        :rtype: str
        """
        return self._column_name

    @column_name.setter
    def column_name(self, column_name):
        """Sets the column_name of this AnalyticsSchemaColumn.

            The name of the column in the table or null if this is a synthetic column which is the result of some SQL expression.

        :param column_name: The column_name of this AnalyticsSchemaColumn.
        :type: str
        """

        self._column_name = column_name
    
    @property
    def description(self):
        """Gets the description of this AnalyticsSchemaColumn.

            A human readable description of the property contained in this column or null if this is a synthetic column which is the result of some SQL expression.

        :return: The description of this AnalyticsSchemaColumn.
        :rtype: dict(str, str)
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this AnalyticsSchemaColumn.

            A human readable description of the property contained in this column or null if this is a synthetic column which is the result of some SQL expression.

        :param description: The description of this AnalyticsSchemaColumn.
        :type: dict(str, str)
        """

        self._description = description
    
    @property
    def precision(self):
        """Gets the precision of this AnalyticsSchemaColumn.

            The precision (maximal number of digits) for decimal data types, otherwise 0.

        :return: The precision of this AnalyticsSchemaColumn.
        :rtype: int
        """
        return self._precision

    @precision.setter
    def precision(self, precision):
        """Sets the precision of this AnalyticsSchemaColumn.

            The precision (maximal number of digits) for decimal data types, otherwise 0.

        :param precision: The precision of this AnalyticsSchemaColumn.
        :type: int
        """

        self._precision = precision
    
    @property
    def referenced_table(self):
        """Gets the referenced_table of this AnalyticsSchemaColumn.

            The name of the referenced table if this column represents a foreign-key relation to the IDs of another table, otherwise null.

        :return: The referenced_table of this AnalyticsSchemaColumn.
        :rtype: str
        """
        return self._referenced_table

    @referenced_table.setter
    def referenced_table(self, referenced_table):
        """Sets the referenced_table of this AnalyticsSchemaColumn.

            The name of the referenced table if this column represents a foreign-key relation to the IDs of another table, otherwise null.

        :param referenced_table: The referenced_table of this AnalyticsSchemaColumn.
        :type: str
        """

        self._referenced_table = referenced_table
    
    @property
    def scale(self):
        """Gets the scale of this AnalyticsSchemaColumn.

            The scale (maximal number number of digits in the fractional part) in case of a decimal data type, otherwise 0.

        :return: The scale of this AnalyticsSchemaColumn.
        :rtype: int
        """
        return self._scale

    @scale.setter
    def scale(self, scale):
        """Sets the scale of this AnalyticsSchemaColumn.

            The scale (maximal number number of digits in the fractional part) in case of a decimal data type, otherwise 0.

        :param scale: The scale of this AnalyticsSchemaColumn.
        :type: int
        """

        self._scale = scale
    
    @property
    def table_name(self):
        """Gets the table_name of this AnalyticsSchemaColumn.

            The name of the table which defines this column.

        :return: The table_name of this AnalyticsSchemaColumn.
        :rtype: str
        """
        return self._table_name

    @table_name.setter
    def table_name(self, table_name):
        """Sets the table_name of this AnalyticsSchemaColumn.

            The name of the table which defines this column.

        :param table_name: The table_name of this AnalyticsSchemaColumn.
        :type: str
        """

        self._table_name = table_name
    
    @property
    def type(self):
        """Gets the type of this AnalyticsSchemaColumn.

            The ORC data type of the column value.

        :return: The type of this AnalyticsSchemaColumn.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this AnalyticsSchemaColumn.

            The ORC data type of the column value.

        :param type: The type of this AnalyticsSchemaColumn.
        :type: str
        """

        self._type = type
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(AnalyticsSchemaColumn, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, AnalyticsSchemaColumn):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other



# coding: utf-8
import pprint
import six
from enum import Enum



class DebtCollectorConfiguration:

    swagger_types = {
    
        'collector': 'int',
        'conditions': 'list[int]',
        'enabled_space_views': 'list[int]',
        'id': 'int',
        'linked_space_id': 'int',
        'name': 'str',
        'planned_purge_date': 'datetime',
        'priority': 'int',
        'skip_review_enabled': 'bool',
        'state': 'CreationEntityState',
        'version': 'int',
    }

    attribute_map = {
        'collector': 'collector','conditions': 'conditions','enabled_space_views': 'enabledSpaceViews','id': 'id','linked_space_id': 'linkedSpaceId','name': 'name','planned_purge_date': 'plannedPurgeDate','priority': 'priority','skip_review_enabled': 'skipReviewEnabled','state': 'state','version': 'version',
    }

    
    _collector = None
    _conditions = None
    _enabled_space_views = None
    _id = None
    _linked_space_id = None
    _name = None
    _planned_purge_date = None
    _priority = None
    _skip_review_enabled = None
    _state = None
    _version = None

    def __init__(self, **kwargs):
        self.discriminator = None
        
        self.collector = kwargs.get('collector', None)
        self.conditions = kwargs.get('conditions', None)
        self.enabled_space_views = kwargs.get('enabled_space_views', None)
        self.id = kwargs.get('id', None)
        self.linked_space_id = kwargs.get('linked_space_id', None)
        self.name = kwargs.get('name', None)
        self.planned_purge_date = kwargs.get('planned_purge_date', None)
        self.priority = kwargs.get('priority', None)
        self.skip_review_enabled = kwargs.get('skip_review_enabled', None)
        self.state = kwargs.get('state', None)
        self.version = kwargs.get('version', None)
        

    
    @property
    def collector(self):
        """Gets the collector of this DebtCollectorConfiguration.

            The collector handles the debt collection case based on the settings of this configuration.

        :return: The collector of this DebtCollectorConfiguration.
        :rtype: int
        """
        return self._collector

    @collector.setter
    def collector(self, collector):
        """Sets the collector of this DebtCollectorConfiguration.

            The collector handles the debt collection case based on the settings of this configuration.

        :param collector: The collector of this DebtCollectorConfiguration.
        :type: int
        """

        self._collector = collector
    
    @property
    def conditions(self):
        """Gets the conditions of this DebtCollectorConfiguration.

            The conditions applied to the collector configuration restricts the application of this configuration onto a particular debt collection case.

        :return: The conditions of this DebtCollectorConfiguration.
        :rtype: list[int]
        """
        return self._conditions

    @conditions.setter
    def conditions(self, conditions):
        """Sets the conditions of this DebtCollectorConfiguration.

            The conditions applied to the collector configuration restricts the application of this configuration onto a particular debt collection case.

        :param conditions: The conditions of this DebtCollectorConfiguration.
        :type: list[int]
        """

        self._conditions = conditions
    
    @property
    def enabled_space_views(self):
        """Gets the enabled_space_views of this DebtCollectorConfiguration.

            The collector configuration is only enabled for the selected space views. In case the set is empty the collector configuration is enabled for all space views.

        :return: The enabled_space_views of this DebtCollectorConfiguration.
        :rtype: list[int]
        """
        return self._enabled_space_views

    @enabled_space_views.setter
    def enabled_space_views(self, enabled_space_views):
        """Sets the enabled_space_views of this DebtCollectorConfiguration.

            The collector configuration is only enabled for the selected space views. In case the set is empty the collector configuration is enabled for all space views.

        :param enabled_space_views: The enabled_space_views of this DebtCollectorConfiguration.
        :type: list[int]
        """

        self._enabled_space_views = enabled_space_views
    
    @property
    def id(self):
        """Gets the id of this DebtCollectorConfiguration.

            A unique identifier for the object.

        :return: The id of this DebtCollectorConfiguration.
        :rtype: int
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this DebtCollectorConfiguration.

            A unique identifier for the object.

        :param id: The id of this DebtCollectorConfiguration.
        :type: int
        """

        self._id = id
    
    @property
    def linked_space_id(self):
        """Gets the linked_space_id of this DebtCollectorConfiguration.

            The ID of the space this object belongs to.

        :return: The linked_space_id of this DebtCollectorConfiguration.
        :rtype: int
        """
        return self._linked_space_id

    @linked_space_id.setter
    def linked_space_id(self, linked_space_id):
        """Sets the linked_space_id of this DebtCollectorConfiguration.

            The ID of the space this object belongs to.

        :param linked_space_id: The linked_space_id of this DebtCollectorConfiguration.
        :type: int
        """

        self._linked_space_id = linked_space_id
    
    @property
    def name(self):
        """Gets the name of this DebtCollectorConfiguration.

            The collector configuration name is used internally to identify a specific collector configuration. For example the name is used within search fields and hence it should be distinct and descriptive.

        :return: The name of this DebtCollectorConfiguration.
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DebtCollectorConfiguration.

            The collector configuration name is used internally to identify a specific collector configuration. For example the name is used within search fields and hence it should be distinct and descriptive.

        :param name: The name of this DebtCollectorConfiguration.
        :type: str
        """
        if name is not None and len(name) > 100:
            raise ValueError("Invalid value for `name`, length must be less than or equal to `100`")

        self._name = name
    
    @property
    def planned_purge_date(self):
        """Gets the planned_purge_date of this DebtCollectorConfiguration.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :return: The planned_purge_date of this DebtCollectorConfiguration.
        :rtype: datetime
        """
        return self._planned_purge_date

    @planned_purge_date.setter
    def planned_purge_date(self, planned_purge_date):
        """Sets the planned_purge_date of this DebtCollectorConfiguration.

            The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.

        :param planned_purge_date: The planned_purge_date of this DebtCollectorConfiguration.
        :type: datetime
        """

        self._planned_purge_date = planned_purge_date
    
    @property
    def priority(self):
        """Gets the priority of this DebtCollectorConfiguration.

            The priority defines the order in which the collector configuration is tried to be applied onto a debt collection case. The higher the value the less likely the configuration is applied on a case.

        :return: The priority of this DebtCollectorConfiguration.
        :rtype: int
        """
        return self._priority

    @priority.setter
    def priority(self, priority):
        """Sets the priority of this DebtCollectorConfiguration.

            The priority defines the order in which the collector configuration is tried to be applied onto a debt collection case. The higher the value the less likely the configuration is applied on a case.

        :param priority: The priority of this DebtCollectorConfiguration.
        :type: int
        """

        self._priority = priority
    
    @property
    def skip_review_enabled(self):
        """Gets the skip_review_enabled of this DebtCollectorConfiguration.

            When the review is skipped there will be no review for cases which use this configuration.

        :return: The skip_review_enabled of this DebtCollectorConfiguration.
        :rtype: bool
        """
        return self._skip_review_enabled

    @skip_review_enabled.setter
    def skip_review_enabled(self, skip_review_enabled):
        """Sets the skip_review_enabled of this DebtCollectorConfiguration.

            When the review is skipped there will be no review for cases which use this configuration.

        :param skip_review_enabled: The skip_review_enabled of this DebtCollectorConfiguration.
        :type: bool
        """

        self._skip_review_enabled = skip_review_enabled
    
    @property
    def state(self):
        """Gets the state of this DebtCollectorConfiguration.

            The object's current state.

        :return: The state of this DebtCollectorConfiguration.
        :rtype: CreationEntityState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this DebtCollectorConfiguration.

            The object's current state.

        :param state: The state of this DebtCollectorConfiguration.
        :type: CreationEntityState
        """

        self._state = state
    
    @property
    def version(self):
        """Gets the version of this DebtCollectorConfiguration.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :return: The version of this DebtCollectorConfiguration.
        :rtype: int
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this DebtCollectorConfiguration.

            The version is used for optimistic locking and incremented whenever the object is updated.

        :param version: The version of this DebtCollectorConfiguration.
        :type: int
        """

        self._version = version
    

    def to_dict(self):
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            elif isinstance(value, Enum):
                result[attr] = value.value
            else:
                result[attr] = value
        if issubclass(DebtCollectorConfiguration, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        return self.to_str()

    def __eq__(self, other):
        if not isinstance(other, DebtCollectorConfiguration):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self == other
